import contextvars
import inspect
import sys
from functools import wraps
from typing import Any, Callable, Optional, Union

from tenacity import RetryCallState, retry, stop_after_attempt, wait_random

from .api import Api, Auth, get_api
from .checkdata import DataResponse
from .core import FieldParser

# 通用装饰器:自动收集参数并调用 API

caller_var = contextvars.ContextVar("caller_name", default="unknown")


def auto_args_call_api(arg: Union[Callable, str, None] = None) -> Callable:
    def decorator(func: Callable, api_name: Optional[str] = None) -> Callable:
        @wraps(func)
        def wrapper(self, *args, **kwargs) -> DataResponse:
            func(self, *args, **kwargs)
            bound_args = inspect.signature(func).bind(self, *args, **kwargs)
            bound_args.apply_defaults()
            arguments = dict(bound_args.arguments)
            arguments.pop("self")
            caller_var.set(func.__name__)
            return self._call_api(api_name or func.__name__, **arguments)

        return wrapper

    if callable(arg):
        return decorator(arg)
    return lambda func: decorator(func, api_name=arg)


class BaseApiClient:
    def __init__(self, filepath: str, auth: Optional[Auth] = None) -> None:
        self.auth = auth
        self.filepath = filepath
        self.API: dict[str, Any] = get_api(self.filepath)

    @retry(
        stop=stop_after_attempt(10),
        wait=wait_random(min=1, max=5),
        before_sleep=lambda state: BaseApiClient.print_retry_info(state),
    )
    def _call_api(self, mmkey: str, **data: Any) -> DataResponse:
        """统一的 API 调用方式

        Args:
            mmkey (str): API 的名称, 来源于json文件的 mmkey, 防止和其他的常用词冲突,命名为 mmkey
            **data (Any): 请求的参数, 这些参数会覆盖 API 配置中的默认值

        """
        api = self.API[mmkey]
        parsed = FieldParser.parse_dict(api)
        checked_data = FieldParser.validate_and_fill_input(parsed, data)
        api_instance = Api(**checked_data)
        method = api_instance.method.upper()
        residual_params = {
            k: v
            for k, v in data.items()
            if v is not None
            and k not in (api_instance.params or {})
            and k not in (api_instance.data or {})
            and k not in ["url", "URL", "skip"]
        }
        if residual_params:
            print(f"❌ 发现多余的参数: {residual_params.keys()}, 请检查 API 配置")
            sys.exit(1)
        if method in ["GET", "POST", "PUT", "DELETE"]:
            return DataResponse(api_instance.result)
        else:
            print("----" * 10)
            print("❌ 无法识别的请求类型,请检查 API 配置")
            print(f"❌ method: {method}")
            print(f"❌ params: {api.get('params')}")
            print(f"❌ data: {api.get('data')}")
            print("----" * 10)
            raise ValueError("❌ 无法识别的请求类型,请检查 API 配置")

    @staticmethod
    def print_retry_info(retry_state: RetryCallState):
        # 获取上下文中的调用者名
        fn_name = caller_var.get()
        args = retry_state.args
        kwargs = retry_state.kwargs
        if kwargs and "files" in kwargs:
            kwargs.pop("files", None)

        exception = (
            retry_state.outcome.exception() if retry_state.outcome is not None else None
        )
        print("---" * 10)
        print("⚠️ 调用失败,准备重试...")
        print(f"🔁 函数: {fn_name}")
        print(f"📥 参数: args={args}")
        print(f"📥 参数: kwargs={kwargs}")
        print(f"💥 异常: {exception}")
        if retry_state.next_action is not None and hasattr(
            retry_state.next_action, "sleep"
        ):
            print(f"⏳ 等待 {retry_state.next_action.sleep:.2f} 秒后重试...\n")
        else:
            print("⏳ 等待时间未知,无法获取 next_action.sleep\n")
        print("---" * 10)



from __future__ import annotations

from typing import Annotated, Any, Literal, Optional

from pydantic import Field, conlist

from .utils.api import Auth
from .utils.baseapiclient import BaseApiClient, auto_args_call_api
from .utils.checkdata import DataResponse


class Share(BaseApiClient):
    def __init__(self, auth: Optional[Auth] = None) -> None:
        super().__init__(filepath="share", auth=auth)

    @auto_args_call_api()
    def create_payment(
        self,
        shareName: str = Field(max_length=35),
        fileIDList: str = Field(max_length=10000),
        payAmount: int = Field(gt=0, le=99),
        resourceDesc: str = Field(max_length=100, default=""),
        isReward: int = Field(default=0, ge=0, le=10),
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """
        创建付费分享链接

        Args:
            shareName (str): 分享链接名称,链接名要小于35个字符且不能包含特殊字符
            fileIDList (str): 分享文件ID列表,以逗号分割,最大只支持拼接100个文件ID,示例:1,2,3
            payAmount (int): 请输入整数|最小金额1元|最大金额99元
            isReward (int): 是否打赏, 0 否, 1 是, 默认 0
            resourceDesc (str): 资源描述
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def create_free(
        self,
        shareName: str,
        shareExpire: Literal[0, 1, 7, 30],
        fileIDList: Annotated[str, Field(max_length=10000)],
        sharePwd: Optional[str] = None,
        trafficSwitch: Optional[Literal[1, 2]] = None,
        trafficLimitSwitch: Optional[Literal[1, 2]] = None,
        trafficLimit: Optional[int] = Field(default=1000, ge=0),
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """
        创建免费分享链接

        Args:
            shareName (str): 分享链接名称,链接名要小于35个字符且不能包含特殊字符
            shareExpire (float): 分享链接有效期天数,该值为枚举, 固定只能填写:1、7、30、0, 填写0时代表永久分享
            fileIDList (str): 分享文件ID列表,以逗号分割,最大只支持拼接100个文件ID,示例:1,2,3
            sharePwd (str): 设置分享链接提取码
            trafficSwitch (int): 免登录流量包开关, 1 关闭免登录流量包, 2 打开免登录流量包
            trafficLimitSwitch (int): 选填	免登录流量限制开关 1 关闭限制 2 打开限制
            trafficLimit (int): 免登陆限制流量 (单位:字节)
            skip (bool): 是否跳过响应数据的模式校验
        """

    def create(self, *args: Any, **kwargs: Any) -> DataResponse:
        """创建分享链接, create_free 方法的别名 ,本质就是调用 create_free 方法(为了尽力保证和url的末端一致性)

        Args:
            args (Any): 传入参数
            kwargs (Any): 传入参数
        """
        return self.create_free(*args, **kwargs)

    @auto_args_call_api()
    def change_share(
        self,
        shareIdList: Annotated[list[int], conlist(int, max_length=100)],
        trafficSwitch: Optional[Literal[1, 2]] = None,
        trafficLimitSwitch: Optional[Literal[1, 2]] = None,
        trafficLimit: Optional[int] = Field(default=1000, ge=0),
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """修改分享链接

        Args:
            shareIdList (int): 分享链接ID列表,数组长度最大为100
            trafficSwitch (Optional[str], optional): 免登录流量包开关 1 关闭免登录流量包, 2 打开免登录流量包
            trafficLimitSwitch (Optional[str], optional): 免登录流量限制开关 1 关闭限制 2 打开限制
            trafficLimit (Optional[float], optional): 免登陆限制流量 (单位:字节)
            skip (bool): 是否跳过响应数据的模式校验, 原因在于没有返回值
        """

    def info(self, *args: Any, **kwargs: Any) -> DataResponse:
        """修改分享链接, change_share 方法的别名 (为了尽力保证和url的末端一致性)

        Args:
            args (Any): 传入参数
            kwargs (Any): 传入参数
        """
        return self.change_share(*args, **kwargs)

    @auto_args_call_api()
    def get_share(
        self,
        limit: int = Field(gt=0, le=100),
        lastShareId: int = 0,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """
        获取分享链接详情

        Args:
            limit (int): 每页分享链接数量,最大不超过100
            lastShareId (int): 翻页查询时需要填写
            skip (bool): 是否跳过响应数据的模式校验
        """

    def list(self, *args: Any, **kwargs: Any) -> DataResponse:
        """获取分享链接详情, get_share 方法的别名 (为了尽力保证和url的末端一致性)

        Args:
            args (Any): 传入参数
            kwargs (Any): 传入参数
        """
        return self.get_share(*args, **kwargs)



from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()
print("----" * 10)
print(pan123.auth)


def test_user():
    # 获取用户信息
    user_info = pan123.user.info(skip=True)
    print("----" * 10)
    print(f"获取用户信息: \n{user_info.data}")



from pathlib import Path

from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()


def test_pan123openapi():
    fname = "图吧工具箱2409安装程序.exe"
    pan123.download(fname, onlyurl=False)
    # 不知道为什么, 网盘上很多一模一样的文件,就是找不到(感觉是频率限制)

    fname = "图吧工具箱2409安装程序.exe"
    assert Path(fname).exists(), f"文件 {fname} 不存在"

    pan123.upload(fname, fname, 0, overwrite=True, duplicate=True)
    print("上传完成")


def test_list_v2():
    # 获取文件列表
    # file_list = pan123.file.list_v2(0, 10, "国科")
    # print(file_list.data)

    fileinfo = pan123.file.list_v2(0, 10)
    print("----" * 10)
    print(f"获取文件信息: \n{fileinfo.data}")


def test_detail():
    pan123 = Pan123openAPI()

    res = pan123.file.detail(fileID=12004772)
    print(res.data)


def test_download():
    fname = "图吧工具箱2409安装程序.exe"
    pan123.download(fname, onlyurl=False)
    assert Path(fname).exists(), f"文件 {fname} 不存在"

    # pan123.upload(fname, fname, 0, overwrite=True, duplicate=True)
    # print("上传完成")


if __name__ == "__main__":
    # test_list_v2()
    test_download()



import random
from pathlib import Path

from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()


def test_oss_upload():
    imgfiles = [
        f
        for f in Path("assets").glob("*")
        if f.is_file() and f.suffix in [".jpg", ".png", ".webp"]
    ]

    imgfile = random.choice(imgfiles)
    print(f"Uploading {imgfile}")

    # imgfile = "assets/0a75f78d9e_亮元周患脖.png"
    res = pan123.upload_oss(imgfile, Path(imgfile).name)
    print(res)
    # 获取图片列表
    res = pan123.oss.list()
    print("----" * 10)
    print("获取图片列表")
    print(res.data)
    # 获取图片的详情
    res = pan123.oss.detail(
        fileID="yk6baz03t0l000d7w33fdduneewjjdg7DIYPAIiOAIaOAvx0DwF="
    )
    print("----" * 10)
    print("获取图片详情")
    print(res)



import re
from pathlib import Path
from typing import Any

from dotenv import find_dotenv, load_dotenv


def load_env():
    """加载 .env 配置(优先项目目录,其次系统目录)"""

    # 在当前目录下查找 .env 文件,找到则返回路径, 否则返回空字符串
    dotenv_path: str = find_dotenv()
    if dotenv_path:
        load_dotenv(dotenv_path, override=True)
    else:
        # 尝试从系统默认目录加载
        system_env_path = Path.home() / ".env.pan123"
        if system_env_path.exists():
            load_dotenv(system_env_path, override=True)
        else:
            return


load_env()


BASE_URL = "https://open-api.123pan.com"
PLATFORM = "open_platform"
HEADERS = {
    # "Authorization": "Bearer " + self.auth.access_token,
    # "Content-Type": "application/json",
    "Platform": PLATFORM,
}

TYPE_MAP = {
    "int": int,
    "float": float,
    "str": str,
    "number": float,
    "string": str,
    "bool": bool,
    "boolean": bool,
    "list": list,
    "dict": dict,
    "object": dict,
    "array": list,
    "any": Any,
    "none": type(None),
    "null": type(None),
}
DEFAULT_BY_TYPE = {
    "int": 0,
    "float": 0.0,
    "str": "",
    "string": "",
    "number": 0.0,
    "bool": False,
    "boolean": False,
    "list": [],
    "dict": {},
}
TEMPLATE_PATTERN = re.compile(r"{{\s*([\w\.]+)\s*}}", re.IGNORECASE)
EMBEDDED_TEMPLATE_PATTERN = re.compile(r"{{{\s*([\w\.]+)\s*}}}", re.IGNORECASE)



from __future__ import annotations

from typing import Any, List, Optional

from .utils.api import Auth
from .utils.baseapiclient import BaseApiClient, auto_args_call_api
from .utils.checkdata import DataResponse


class Oss(BaseApiClient):
    def __init__(self, auth: Optional[Auth] = None) -> None:
        super().__init__(filepath="oss", auth=auth)

    @auto_args_call_api("mkdir")
    def mkdir(
        self, name: str, parentID: str, type: int = 1, skip: bool = False
    ) -> DataResponse:  # type: ignore
        """创建目录

        Args:
            name (str): 目录名(注:不能重名)
            parentID (str): 父目录id,上传到根目录时为空
            type (int): 固定为 1,
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass

    @auto_args_call_api()
    def create(
        self,
        parentFileID: str,
        filename: str,
        etag: str,
        size: float,
        type: int = 1,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """创建文件

        Args:
            parentFileID (str): 父目录id,上传到根目录时填写 空
            filename (str): 文件名要小于255个字符且不能包含以下任何字符:"\\/:*?|><. (注:不能重名)
            etag (str): 文件md5
            size (int): 文件大小,单位为 byte 字节
            type (int): 固定为 1
            skip (bool): 是否跳过响应数据的模式校验

        """
        pass

    @auto_args_call_api("get_upload_url")
    def get_upload_url(
        self, preuploadID: str, sliceNo: int, skip: bool = False
    ) -> DataResponse:  # type: ignore
        """获取上传地址&上传分片

        Args:
            preuploadID (str): 预上传ID
            sliceNo (int): 分片序号,从1开始自增
            skip (bool): 是否跳过响应数据的模式校验

        """
        pass

    @auto_args_call_api("upload_complete")
    def upload_complete(self, preuploadID: str, skip: bool = False) -> DataResponse:  # type: ignore
        """上传完毕

        Args:
            preuploadID (str): 预上传ID
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass

    @auto_args_call_api("upload_async_result")
    def upload_async_result(self, preuploadID: str, skip: bool = False) -> DataResponse:  # type: ignore
        """异步轮询获取上传结果

        Args:
            preuploadID (str): 预上传ID
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass

    @auto_args_call_api("copy")
    def copy(
        self,
        fileIDs: List[str],
        toParentFileID: str,
        sourceType: str,
        type: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """创建复制任务

        说明:图床复制任务创建(可创建的任务数:3,fileIDs 长度限制:100,当前一个任务处理完后将会继续处理下个任务)

        该接口将会复制云盘里的文件或目录对应的图片到对应图床目录,每次任务包含的图片总数限制 1000 张,图片格式:png, gif, jpeg, tiff, webp,jpg,tif,svg,bmp,图片大小限制:100M,文件夹层级限制:15层

        如果图床目录下存在相同 etag、size 的图片将会视为同一张图片,将覆盖原图片


        Args:
            fileIDs (List[str]): 文件id数组(string 数组)
            toParentFileID (str): 要移动到的图床目标文件夹id,移动到根目录时为空
            sourceType (str): 复制来源(1=云盘)
            type (int): 业务类型,固定为 1
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass

    @auto_args_call_api("process")
    def process(self, taskID: str, skip: bool = False) -> DataResponse:  # type: ignore
        """获取复制任务详情

        说明:该接口将会获取图床复制任务执行情况

        Args:
            taskID (str): 复制任务ID
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass

    @auto_args_call_api("fail")
    def fail(
        self, taskID: str, limit: int, page: int, skip: bool = False
    ) -> DataResponse:  # type: ignore
        """获取复制失败文件列表

        说明:查询图床复制任务失败文件列表(注:记录的是符合对应格式、大小的图片的复制失败原因)

        Args:
            taskID (str): 复制任务ID
            limit (int): 每页文件数量,最大不超过100
            page (int): 页码数
            skip (bool): 是否跳过响应数据的模式校验
        """
        if limit > 100:
            raise ValueError("❌ limit must be less than 100")
        pass

    @auto_args_call_api("move")
    def move(self, fileIDs: List[str], toParentFileID: str, skip: bool = False) -> None:
        """移动文件或目录

        Args:
            fileIDs (List[str]): 文件id数组(string 数组)
            toParentFileID (str): 要移动到的目标文件夹id,移动到根目录时填写 空
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass

    @auto_args_call_api("delete")
    def delete(self, fileIDs: List[str]) -> None:
        """删除文件或目录

        Args:
            fileIDs (List[str]): 文件id数组,参数长度最大不超过 100
        """
        if len(fileIDs) > 100:
            raise ValueError("❌ fileIDs length must be less than 100")

    @auto_args_call_api()
    def detail(self, fileID: str) -> DataResponse | None:
        """获取文件或目录详情

        Args:
            fileID (str): 文件ID
        """
        pass

    @auto_args_call_api()
    def file_list(
        self,
        parentFileId: Optional[str] = "",
        limit: int = 20,
        startTime: Optional[int] = None,
        endTime: Optional[int] = None,
        lastFileId: Optional[str] = None,
        type: int = 1,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取图片列表

        Args:
            parentFileId (str): 父级目录Id, 默认为空表示筛选根目录下的文件
            limit (int): 每页文件数量,最大不超过100
            startTime (int, optional): 筛选开始时间(时间戳格式,例如 1730390400)
            endTime (int, optional): 筛选结束时间(时间戳格式,例如 1730390400)
            lastFileId (str, optional): 上翻页查询时需要填写
            type (int): 业务类型,固定为 1
            skip (bool): 是否跳过响应数据的模式校验
        """
        if limit > 100:
            raise ValueError("❌ imit must be less than 100")

    def list(self, *args: Any, **kwargs: Any) -> DataResponse:
        """获取图片列表

        等价与 self.file_list()

        Args:
            args (Any): 传入参数
            kwargs (Any): 传入参数
        """
        return self.file_list(*args, **kwargs)

    @auto_args_call_api()
    def offline_download(
        self,
        url: str,
        fileName: Optional[str] = None,
        businessDirID: Optional[str] = None,
        callBackUrl: Optional[str] = None,
        type: int = 1,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """离线下载

        Args:
            url (str): 下载资源地址(http/https)
            fileName (str, optional):  自定义文件名称 (需携带图片格式,支持格式:png, gif, jpeg, tiff, webp,jpg,tif,svg,bmp)
            businessDirID (str, optional): 选择下载到指定目录ID.  示例:10023. 注:不支持下载到根目录,默认下载到名为"来自:离线下载"的目录中
            callBackUrl (str, optional): 回调地址,当文件下载成功或者失败,均会通过回调地址通知
            type (int): 业务类型,固定为 1
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def offline_process(self, taskID: int, skip: bool = False) -> DataResponse:  # type: ignore
        """获取离线迁移任务

        说明:获取当前离线下载任务状态

        Args:
            taskID (int):离线下载任务ID
            skip (bool): 是否跳过响应数据的模式校验
        """
        pass



from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()


def test_pan123openapi():
    #
    res = pan123.file.list(0, 100)
    print("----" * 10)
    print(f"获取文件列表: \n{res}")
    print("----" * 10)
    # 查找 图吧工具箱2409安装程序( 对应的fileID, 不要删除本体
    fileid = None
    assert res.data, "没有获取到文件列表"
    for item in res.data["fileList"]:
        if "图吧工具箱2409安装程序(" in item["filename"] and item["trashed"] == 0:
            fileid = item["fileId"]
            filename = item["filename"]
            break
    assert fileid, "没有找到文件ID"
    print("----" * 10)
    print(f"文件ID: {fileid}")
    print(f"文件名: {filename}")
    res = pan123.file.trash(fileIDs=[fileid])
    print("----" * 10)
    print(f"删除文件: \n{res}")
    print("----" * 10)



import hashlib
from pathlib import Path
from typing import Optional


def calculate_md5(file_path: Path | str) -> str:
    """计算文件的 MD5 值

    Args:
        file_path (Path | str): 文件路径,可以是 Path 对象或字符串

    Returns:
        str: 文件的 MD5 值

    Raises:
        AssertionError: 文件不存在或路径不是文件或文件大小为0

    """
    if isinstance(file_path, str):
        file_path = Path(file_path)

    assert file_path.exists(), f"❌ 文件不存在: {file_path}"
    assert file_path.is_file(), f"❌ 路径不是文件: {file_path}"
    hash_md5 = hashlib.md5()
    with file_path.open("rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)
    return hash_md5.hexdigest()


def get_file_md5_blocks(file_path, block_size=32 * 1024 * 1024):
    file_path = Path(file_path)
    if not file_path.exists():
        raise FileNotFoundError(f"File not found: {file_path}")

    block_list = []
    with file_path.open("rb") as f:
        while True:
            chunk = f.read(block_size)
            if not chunk:
                break
            md5 = hashlib.md5(chunk).hexdigest()
            block_list.append(md5)

    return block_list


def encrypt_md5(md5str):
    if len(md5str) != 32:
        return md5str
    for i in range(0, 32):
        v = int(md5str[i], 16)
        if v < 0 or v > 16:
            return md5str
    md5str = md5str[8:16] + md5str[0:8] + md5str[24:32] + md5str[16:24]
    encryptstr = ""
    for e in range(0, len(md5str)):
        encryptstr += hex(int(md5str[e], 16) ^ 15 & e)[2:3]
    return encryptstr[0:9] + chr(ord("g") + int(encryptstr[9], 16)) + encryptstr[10:]


def calculate_slice_md5(file_path):
    """
    计算文件前 256KB 的 MD5 值.

    :param file_path: 文件路径
    :return: 32 位小写的 MD5 字符串
    """
    md5 = hashlib.md5()
    try:
        with open(file_path, "rb") as f:
            data = f.read(256 * 1024)  # 读取前 256KB
            md5.update(data)
        return md5.hexdigest()
    except FileNotFoundError:
        print(f"文件未找到: {file_path}")
        return None
    except Exception as e:
        print(f"读取文件时发生错误: {e}")
        return None


def decrypt_md5(encrypted):
    if len(encrypted) != 32:
        return encrypted

    # Step 1: 恢复 encryptstr(含替代位)
    g_digit = ord(encrypted[9]) - ord("g")
    encryptstr = encrypted[:9] + hex(g_digit)[2:] + encrypted[10:]

    # Step 2: 逆异或
    md5_mixed = ""
    for e in range(len(encryptstr)):
        v = int(encryptstr[e], 16)
        orig = v ^ (e & 15)
        md5_mixed += hex(orig)[2:]

    # Step 3: 逆重排
    if len(md5_mixed) != 32:
        return "无法还原:长度不符"

    part1 = md5_mixed[8:16]  # 原 0-7
    part2 = md5_mixed[0:8]  # 原 8-15
    part3 = md5_mixed[24:32]  # 原 16-23
    part4 = md5_mixed[16:24]  # 原 24-31

    recovered_md5 = part1 + part2 + part3 + part4
    return recovered_md5


def calculate_sha256(file_path: str) -> str:
    """
    计算文件的 SHA256 值.
    """
    sha256_hash = hashlib.sha256()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(8192), b""):
            sha256_hash.update(chunk)
    return sha256_hash.hexdigest()


def calculate_sha1(file_path: str) -> str:
    """
    计算文件的 SHA1 值.
    """
    sha1_hash = hashlib.sha1()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(8192), b""):
            sha1_hash.update(chunk)
    return sha1_hash.hexdigest()


def calculate_sha512(file_path: str) -> str:
    """
    计算文件的 SHA512 值.
    """
    sha512_hash = hashlib.sha512()
    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(8192), b""):
            sha512_hash.update(chunk)
    return sha512_hash.hexdigest()


def check_hash(
    file_path: str,
    expected_md5: Optional[str] = None,
    expected_sha1: Optional[str] = None,
    expected_sha256: Optional[str] = None,
    expected_sha512: Optional[str] = None,
) -> bool:
    """
    校验文件的哈希值.  通过则返回 True, 否则返回 False.

    Args:
        file_path (str): 文件路径.
        expected_md5 (str): 预期的 MD5 值.
        expected_sha1 (str): 预期的 SHA1 值.
        expected_sha256 (str): 预期的 SHA256 值.
        expected_sha512 (str): 预期的 SHA512 值.
    Returns:
        bool: 校验结果, True 表示通过, False 表示失败.
    """
    if not Path(file_path).exists():
        raise FileNotFoundError(f"文件不存在: {file_path}")

    if not Path(file_path).is_file():
        raise ValueError(f"路径不是文件: {file_path}")
    if (
        not expected_md5
        and not expected_sha256
        and not expected_sha1
        and not expected_sha512
    ):
        # 如果没有提供任何校验值, 则直接返回 True
        return True
    if expected_md5:
        actual_md5 = calculate_md5(file_path)
        if actual_md5.lower() == expected_md5.lower():
            return True
    if expected_sha256:
        actual_sha256 = calculate_sha256(file_path)
        if actual_sha256.lower() == expected_sha256.lower():
            return True
    if expected_sha1:
        actual_sha1 = calculate_sha1(file_path)
        if actual_sha1.lower() == expected_sha1.lower():
            return True
    if expected_sha512:
        actual_sha512 = calculate_sha512(file_path)
        if actual_sha512.lower() == expected_sha512.lower():
            return True
    return False



import importlib.resources as pkg_resources
import sys
import time
import warnings
from pathlib import Path
from typing import Any, Dict, Optional, Union

import json5
import requests
from jsonpath import jsonpath
from jsonschema import validate
from pydantic import Field, TypeAdapter, dataclasses

from .auth import Auth
from .checkdata import BaseResponse, JsonInput
from .const import BASE_URL, HEADERS, TEMPLATE_PATTERN


def get_api(filepath: str, *args: Any) -> dict:
    """
    获取 API.

    Args:
        filepath (str): API 所属分类,即 `apijson/***.json`下的文件名(不含后缀名)
        *args (Any): 预留的可选参数(当前未使用).

    Returns:
        dict, 该 API 的内容.
    """
    path = Path(filepath)
    # 如果没有后缀,则添加.json后缀
    if not Path(filepath).suffix:
        path = path.with_suffix(".json")

    # 如果是相对路径,则在当前目录下查找
    # 处理相对路径
    if not path.is_absolute():
        try:
            path = pkg_resources.files("cpan123.apijson").joinpath(str(path))
            path = Path(str(path))
        except ModuleNotFoundError:
            print("❌ 找不到模块 `cpan123.apijson`,请确认路径或依赖包正确")
            sys.exit(1)

    if not path.exists():
        print(f"❌ 文件不存在: {path}")
        sys.exit(1)

    # 读取并校验 JSON 文件
    with open(path, "r", encoding="utf-8") as file:
        try:
            data: dict = json5.load(file)
        except Exception as e:
            print(f"❌ JSON 解析失败: {path}\n错误: {e}")
            sys.exit(1)

    list_adapter = TypeAdapter(Dict[str, JsonInput])
    list_adapter.validate_python(data)
    # 按参数索引嵌套数据
    for arg in args:
        try:
            data = data[arg]
        except KeyError:
            print(f"❌ 参数 `{arg}` 不存在于 API 数据中")
            sys.exit(1)
    return data


@dataclasses.dataclass
class Api:
    """
    用于请求的 Api 类
    """

    method: str
    url: str
    data: Optional[dict] = Field(default_factory=dict)
    params: Optional[dict] = Field(default_factory=dict)
    response_schema: Optional[dict] = Field(default_factory=dict)
    schema_: Optional[dict] = Field(default_factory=dict)
    comment: str = ""
    auth: Auth = Field(default_factory=Auth)
    headers: dict = Field(default_factory=dict)
    files: Optional[Any] = None
    skip: bool = Field(default=False)

    def __post_init__(self) -> None:
        """
        初始化 Api 对象
        """
        # 获取请求方法
        self.method = self.method.upper()
        self.data = self.data or None
        self.params = self.params or None
        self.response_schema = self.response_schema or None
        self.schema_ = self.schema_ or None
        self.auth = self.auth or Auth()
        self.headers = self.headers or HEADERS.copy()
        self.files = self.files or None

    def _update_attr(self, attr: str, **kwargs) -> "Api":
        if "skip" in kwargs:
            self.skip = kwargs.pop("skip")
        value = {k: v for k, v in kwargs.items() if v is not None}
        # 如果 v 是 list,则将其转换为 str
        for k, v in value.items():
            if isinstance(v, list):
                value[k] = json5.dumps(v, ensure_ascii=False, separators=(",", ":"))
            elif isinstance(v, dict):
                value[k] = json5.dumps(v, ensure_ascii=False, separators=(",", ":"))
        setattr(self, attr, value)
        return self

    def update_attr2(self) -> "Api":
        def stringify_values(obj):
            if isinstance(obj, dict):
                return {
                    k: json5.dumps(v, ensure_ascii=False, separators=(",", ":"))
                    if isinstance(v, (dict, list))
                    else v
                    for k, v in obj.items()
                }
            return obj  # 保持原样（非 dict 不处理）

        self.data = stringify_values(self.data)
        self.params = stringify_values(self.params)

        return self

    def update_data(self, **kwargs) -> "Api":
        return self._update_attr("data", **kwargs)

    def update_params(self, **kwargs) -> "Api":
        return self._update_attr("params", **kwargs)

    def update_files(self, arg) -> "Api":
        self.files = arg
        return self

    def update_method(self, method: str) -> "Api":
        self.method = method.upper()
        return self

    def update_url(self, url: str) -> "Api":
        self.url = url
        return self

    def update_headers(self, **kwargs) -> "Api":
        self.headers = kwargs
        return self

    def _resolve_key_path(self, data: Any, key_path: str) -> Any:
        """
        解析嵌套的 key 路径，支持 {{ key }} 和 {{ key.key2 }} 的格式。
        优先支持字典和 Auth 对象属性查找，未找到时保留原模板字符串。
        """
        keys = key_path.split(".")
        val = data
        for key in keys:
            if isinstance(val, dict):
                val = val.get(key)
            elif hasattr(val, key):
                val = getattr(val, key)
            else:
                return f"{{{{ {key_path} }}}}"  # fallback: 保留模板格式
        return val if val is not None else f"{{{{ {key_path} }}}}"

    def resolve_templates(self, data: Any) -> Any:
        """
        递归解析模板字符串，支持 {{ key }} 和 {{ a.b.c }} 格式，处理字符串、字典和列表。
        """

        def _template(string: str) -> str:
            return TEMPLATE_PATTERN.sub(
                lambda match: str(self._resolve_key_path(self.auth, match.group(1))),
                string,
            )

        if isinstance(data, str):
            return _template(data)
        elif isinstance(data, dict):
            return {k: self.resolve_templates(v) for k, v in data.items()}
        elif isinstance(data, list):
            return [self.resolve_templates(v) for v in data]
        return data

    def _prepare_request(self) -> dict:
        """
        准备请求参数
        """
        self.update_attr2()
        headers = self.headers.copy()
        if not self.files:
            headers["Content-Type"] = "application/json"
        headers["Authorization"] = f"Bearer {self.auth.access_token}"

        full_url = (
            self.url
            if self.url.startswith("http")
            else f"{BASE_URL.rstrip('/')}/{self.url.lstrip('/')}"
        )
        config = {
            "method": self.method,
            "url": full_url,
            "params": self.params,
            "data": self.data,
            "files": self.files,
            "headers": headers,
        }

        config = {k: v for k, v in config.items() if v is not None}
        if config.get("files") is not None:
            # 如果有文件上传,则不需要设置 Content-Type
            # 因为 requests 会自动设置
            config["headers"].pop("Content-Type", None)
        # 解析模板
        config = self.resolve_templates(config)

        return config

    def request(self, byte: bool = False) -> Union[int, str, dict, bytes, None]:
        """
        发送请求并返回结果

        Args:
            byte (bool): 是否返回字节流,默认为 False
        """
        # 处理请求参数
        config: dict = self._prepare_request()
        # print("---" * 10)
        # print(f"🔍 请求参数: {config}")
        for _ in range(3):
            response = requests.request(**config)
            # print("response.url:", response.url)
            response.raise_for_status()
            code = response.json().get("code", None)
            if code == 429:
                # todo: 需要根据不同的 code 进行处理
                # 请求太频繁
                time.sleep(3)
            elif code == 401:
                self.auth.refresh_access_token()
                # 权益已过期
            else:
                break

        if byte:
            return response.text

        if self.skip:
            # 如果不需要验证响应数据的 schema_,则直接返回
            return response.json()
        if self.schema_:
            res_json: dict = response.json()
            BaseResponse.model_validate(res_json)  # 校验响应数据的格式
            # 由于 123 运行返回的数据都有 data,因此,只需要校验 data 字段
            if res_json.get("data"):
                validate(instance=res_json["data"], schema=self.schema_)
                return response.json()
            else:
                print(f"❌ 响应数据: {response.text}")
                print(f"❌ 响应数据: {res_json}")
                if res_json.get("code") == 401:
                    print(f"❌ {res_json.get('message')}")
                    sys.exit(1)
                raise ValueError("❌ 利用 schema_ 校验失败, 没有 data 字段")

        if self.response_schema:
            check = self.validate_response_schema(response, self.response_schema)
            if check:
                return response.json()
            else:
                raise ValueError("❌ 利用 response_schema 校验失败")

        return response.json()

    @property
    def result(self) -> dict:
        res = self.request()
        if isinstance(res, dict):
            return res
        else:
            raise ValueError(f"❌ 响应数据解析失败: {res}")

    @staticmethod
    def validate_response_schema(response: requests.Response, schema_: dict) -> bool:
        if not schema_:
            return True

        try:
            response.raise_for_status()
            res_json = response.json()
            BaseResponse.model_validate(res_json)
        except Exception as e:
            print(f"❌ 响应数据解析失败: {response.text}\n错误: {e}")
            return False

        type_mapping = {
            "string": str,
            "number": (int, float),
            "int": int,
            "float": float,
            "boolean": bool,
            "bool": bool,
            "object": dict,
            "array": list,
        }

        errors = []
        # 校验响应数据, 和普通的 jsonschema 校验不同的是,这里的 schema_ 是一个 dict
        # 比如 { "key": "string" } 代表 key 的值是 string 类型
        # 比如 {"key": "int"} 代表 key 的值是 int 类型
        for k, rule in schema_.items():
            expected_type = type_mapping.get(rule["type"])
            actual = jsonpath(res_json, f"$..{k}")
            if not actual:
                errors.append(f"Key '{k}' 不存在于响应中")
                continue
            if expected_type and not isinstance(actual[0], expected_type):
                errors.append(f"Key '{k}' 预期类型: {rule['type']}")

        if errors:
            for error in errors:
                print(f"❌ {error}")
            warnings.warn(f"❌ 校验失败响应: {res_json}", stacklevel=2)
            return False

        return True


__all__ = [
    "get_api",
    "Api",
    "Auth",
]



"""获取123网盘的访问令牌

采用 env 文件来管理授权信息,

- 项目级别的配置文件: `.env`
- 系统级别的配置文件: `~/.env.pan123`

如果两个文件都存在, 则优先使用项目级别的配置文件

需要去[123云盘开发平台](https://www.123pan.com/developer) 申请一个应用

配置文件参考如下
```
PAN123CLIENTID="***"
PAN123CLIENTSECRET="***"
```
当有了以上信息以后, 运行下面的代码, 会自动生成一个访问令牌, 并保存到 `.env` 文件中
```python
from cpan123 import Auth
auth = Auth()
print(auth) # 打印访问令牌

### 访问令牌会自动保存到 .env 文件中
PAN123TOKEN="**"
PAN123TOKEN_EXPIREDAT="***"
```


"""

import os
import sys
from datetime import datetime
from pathlib import Path
from typing import Optional

import requests
from dotenv import set_key
from pydantic import dataclasses
from tenacity import retry, stop_after_attempt, wait_random

from .const import BASE_URL, PLATFORM, load_env


@dataclasses.dataclass
class Auth:
    """用于获取和设置访问令牌的类.

    在每次访问 `.token` 时,会自动检查令牌是否存在或过期, 且提供了 clientID/clientSecret,就会自动刷新.  而`.access_token` 不会,是固定的值

    Attributes:
        clientID (str): 客户端 ID
        clientSecret (str): 客户端密钥
        access_token (str): 访问令牌
        access_token_expiredAt (str): 访问令牌过期时间
        token (str): 访问令牌, 每次访问时自动检查是否过期并刷新

    Example:
    ```python
    from cpan123 import Auth
    auth = Auth()
    print(auth.access_token)
    # 如果什么都没有设置, 则会自动从环境变量中加载
    # 优先从项目中目录下的 .env 文件中获取, 其次从系统目录下的 .env.pan123 文件中获取
    # 如果都没有,则会抛出异常

    # 或者手动设置
    from cpan123 import Auth
    auth = Auth(clientID="your_client_id", clientSecret="your_client_secret")
    print(auth.access_token) # 打印固定值
    # 强制刷新
    auth.refresh_access_token()
    # 会自动检查是否过期,尝试自动刷新
    print(auth.token)

    # 或者
    auth = Auth(access_token="your_access_token")
    print(auth.access_token)
    ```
    """

    clientID: Optional[str] = None
    clientSecret: Optional[str] = None

    access_token: Optional[str] = None
    access_token_expiredAt: Optional[str] = None  # ISO8601格式字符串

    def __post_init__(self) -> None:
        """
        初始化 Auth 对象
        """
        self._load_from_env()
        # 自动获取 access_token(如果未提供,但提供了 clientID 和 clientSecret)
        if not self.access_token:
            if self.clientID and self.clientSecret:
                self.refresh_access_token()
            else:
                raise ValueError("❌ No access token or client credentials found")

    def _load_from_env(self):
        """从环境变量加载认证信息"""
        load_env()
        self.clientID = self.clientID or os.getenv("PAN123CLIENTID")
        self.clientSecret = self.clientSecret or os.getenv("PAN123CLIENTSECRET")
        self.access_token = self.access_token or os.getenv("PAN123TOKEN")
        self.access_token_expiredAt = self.access_token_expiredAt or os.getenv(
            "PAN123TOKEN_EXPIREDAT"
        )

    def _is_token_expired(self) -> bool:
        """判断 access_token 是否过期"""
        if not self.access_token_expiredAt:
            return False  # 没有过期时间,认为没过期
        try:
            expire_dt = datetime.fromisoformat(self.access_token_expiredAt)
            now = datetime.now()
            return now >= expire_dt
        except ValueError:
            print("❌ Invalid access_token_expiredAt format")
            return True

    @property
    def token(self) -> str | None:
        """
        每次访问时自动检查是否过期并刷新
        """
        if self._is_token_expired():
            if self.clientID and self.clientSecret:
                print("🔁 Token 已过期,正在刷新...")
                self.refresh_access_token()
            else:
                print("⚠️ Token 已过期,缺少 clientID/clientSecret,无法刷新, 退出程序")
                sys.exit(1)

        return self.access_token

    def set_access_token(self, access_token: str) -> "Auth":
        """
        设置 access_token

        Args:
            access_token (str): 访问令牌
        """
        self.access_token = access_token
        self.access_token_expiredAt = None
        return self

    @retry(stop=stop_after_attempt(3), wait=wait_random(min=1, max=5))
    def refresh_access_token(self) -> "Auth":
        """重新获取 access_token, 或手动设置 access_token. 用于强制刷新

        强制刷新 access_token, 前提是存在 clientID 和 clientSecret.

        - 如果不存在且没有 access_token, 则退出
        - 如果不存在且有 access_token, 则不刷新access_token, 打印警告

        """
        if not self.access_token:
            if not (self.clientID and self.clientSecret):
                print("❌ No clientID/clientSecret found, and no access_token, exiting")
                sys.exit(1)
            # 有 clientID 和 clientSecret,但没有 token → 尝试刷新
            self.refresh_access_token()
        else:
            if not (self.clientID and self.clientSecret):
                print("⚠️ access_token 存在,但未提供 clientID/clientSecret,跳过刷新")
                return self
        try:
            response = requests.post(
                BASE_URL + "/api/v1/access_token",
                data={"clientID": self.clientID, "clientSecret": self.clientSecret},
                headers={"Platform": PLATFORM},
                timeout=60,
            )
            response.raise_for_status()
            data = response.json().get("data", {})
            self.access_token = data.get("accessToken")
            self.access_token_expiredAt = data.get("expiredAt")
        except Exception as e:
            print(f"❌ 请求 access_token 接口失败: {e}")
            raise

        if not self.access_token or not self.access_token_expiredAt:
            raise ValueError("❌ 获取 access_token 响应数据不完整")

        self.save_info()
        return self

    def save_info(self):
        """将 access_token 及相关信息保存到 .env 文件"""
        project_env = Path(".env")
        system_env = Path.home() / ".env.pan123"

        if project_env.exists():
            env_path = project_env
        elif system_env.exists():
            env_path = system_env
        else:
            # 默认创建项目目录下的 .env
            env_path = project_env
            env_path.touch()

        if self.clientID:
            set_key(env_path, "PAN123CLIENTID", self.clientID)
        if self.clientSecret:
            set_key(env_path, "PAN123CLIENTSECRET", self.clientSecret)
        if self.access_token:
            set_key(env_path, "PAN123TOKEN", self.access_token)
        if self.access_token_expiredAt:
            set_key(env_path, "PAN123TOKEN_EXPIREDAT", self.access_token_expiredAt)
        print(f"✅ 认证信息已保存到 {env_path} 文件中")



from __future__ import annotations

from typing import Optional

from .utils.api import Auth
from .utils.baseapiclient import BaseApiClient, auto_args_call_api
from .utils.checkdata import DataResponse


class User(BaseApiClient):
    def __init__(self, auth: Optional[Auth] = None) -> None:
        super().__init__(filepath="user", auth=auth)

    @auto_args_call_api()
    def info(self, skip=False) -> DataResponse:  # type: ignore
        """
        获取用户信息
        """



import math
import os
import time
import warnings
from concurrent.futures import ThreadPoolExecutor, as_completed
from pathlib import Path, PurePosixPath
from typing import Any, BinaryIO, Callable, Iterable, List, Optional, Union

import requests
from py3_wget.main import download_file
from pydantic import BaseModel, TypeAdapter, validate_call
from tenacity import retry, stop_after_attempt, wait_random

from .directlink import DirectLink
from .file import Auth, File
from .offline import Offline
from .oss import Oss
from .share import Share
from .user import User
from .utils.checkdata import UploadInChunks

# 定义允许的单个类型
SingleDirType = Union[str, int, Path, PurePosixPath]

# 定义参数类型,可以是单个值或这些值的序列对象
DirnamesType = Union[SingleDirType, Iterable[SingleDirType]]


class FileItem(BaseModel):
    fileId: int
    filename: str
    full_path: str
    relative_path: str
    etag: str
    model_config = {"extra": "allow"}


class Pan123openAPI:
    """封装好的一些常用的 API 接口, 方便使用

    1. 文件上传
    2. 文件下载
    3. 图片上传到 OSS

    授权信息参考: [Auth](./auth.md)

    Attributes:
        auth (Auth): 授权对象
        file (File): 文件对象
        share (Share): 分享对象
        offline (Offline): 离线下载对象
        user (User): 用户对象
        directlink (DirectLink): 直链对象
        oss (Oss): OSS 对象

    Example:
        ```python
        from cpan123 import Pan123openAPI
        pan123 = Pan123openAPI()

        # 下载单个文件
        fname = "xxxx.pt"
        pan123.download(fname)

        # 上传文件
        fname = "xxxxx.zip"
        pan123.upload(fname, fname, 0, overwrite=True)

        # 上传目录
        dirname = Path("tdata")
        pan123.upload_dir(dirname)

        # 列出目录下的所有文件(递归且包含目录本身)
        dirname = Path("/tdata")
        files = pan123.list_files(dirname)

        # 下载目录(从网盘/data 目录 下载到本地 tdata1 目录)
        dirname = Path("/tdata")
        pan123.download_dir(dirname, "tdata1")

        # 删除目录
        dirname = Path("/tdata")
        pan123.delete_dir(dirname)

        # 根据路径查找文件 ID以及文件信息
        file = "/xxx/xxx.zip"
        fileId, fileId_item = pan123.find_file_id_by_path(file, is_dir=False)
        ```
    """

    def __init__(
        self,
        auth: Optional[Auth] = None,
    ):
        self.auth = auth if auth is not None else Auth()
        if not isinstance(self.auth, Auth):
            raise ValueError("auth 必须是 Auth 对象")
        self.file = File(auth=self.auth)
        self.share = Share(auth=self.auth)
        self.offline = Offline(auth=self.auth)
        self.user = User(auth=self.auth)
        self.directlink = DirectLink(auth=self.auth)
        self.oss = Oss(auth=self.auth)

    def _validate_and_prepare_paths(
        self,
        filename: Union[str, Path, PurePosixPath],
        upload_name: Union[str, Path, PurePosixPath, None] = None,
    ) -> tuple[str, str]:
        """验证和准备文件路径(本地路径和上传路径)

        1. 如果本地文件不存在,则抛出异常
        2. 如果 upload_name 为 None,则使用本地文件名, 否则使用 upload_name定义的文件名

        Args:
            filename (Union[str, Path, PurePosixPath]): 本地文件路径
            upload_name (Union[str, Path, PurePosixPath, None]): 上传到云端的文件名. 如果为 None,则使用本地文件名
        """

        filename = Path(filename) if not isinstance(filename, Path) else filename
        upload_name = Path(upload_name) if upload_name else Path(filename).name

        if not filename.exists():
            raise ValueError(f"❌ 本地文件 {filename} 不存在")

        return str(filename), str(upload_name)

    @validate_call
    def download(
        self,
        filename: Union[str, Path, List[Union[str, Path]]],
        onlyurl: bool = False,
        overwrite: bool = False,
    ) -> List[dict] | None:
        """根据文件名下载文件(不覆盖).

        采用的是 `pip install py3-wget` 作为下载工具,支持支持进度条、校验和验证、超时处理和下载失败时自动重试.

        已添加参数:

        - md5: 文件的 md5 值
        - max_tries: 最大重试次数 (默认 5 次)
        - retry_seconds: 重试间隔时间(秒) (默认 2 秒)


        Args:
            filename (Union[str, Path, List[Union[str, Path]]]): 单个文件名或文件名列表, 文件名会被存储到本地文件,如果本地有同名文件, 则会报错,可选强制覆盖, 如果使用相对路径, 则默认在根目录下
            onlyurl (bool): 是否只获取下载链接. 默认直接下载文件.
            overwrite (bool): 是否覆盖同名文件. 默认 False.

        Returns:
            若 onlyurl 为 True, 返回下载链接列表; 否则返回 None.

        Raises:
            ValueError: 当输入文件名无效时


        Example:
            ```python
            from cpan123 import Pan123openAPI
            pan123 = Pan123openAPI()

            # # 下载单个文件
            fname = "xxxx.pt"
            pan123.download(fname)

            # # 下载多个文件
            # filenames = ["xxxx1.pt", "xxx2.pt"]
            # pan123.download(filenames)
            ```
        """
        # 参数标准化和验证
        filename = [filename] if isinstance(filename, (str, Path)) else filename
        if not isinstance(filename, list):
            raise ValueError("filename 必须是字符串或 Path 的列表")

        if not all(isinstance(fname, (str, Path)) for fname in filename):
            raise ValueError("filename 必须是字符串或 Path 的列表")

        # 检查本地是否存在该文件
        need_down_files = []
        for fname in filename:
            if Path(fname).exists():
                if not overwrite:
                    print(f"⚠️ 文件 {fname} 已存在,跳过下载")
                    continue
                Path(fname).unlink()
            need_down_files.append(fname)

        # 下载进度跟踪
        if not need_down_files:
            print("⚠️ 没有需要下载的文件")
            return None

        results = []
        need_down_files = [Path(fname) for fname in need_down_files]
        for _idx, fname in enumerate(need_down_files):
            try:
                # # 构造云端路径
                if not fname.is_absolute():
                    fname = PurePosixPath("/") / str(fname).lstrip("./")

                if not fname.suffix:
                    # 假设没有后缀名的文件是目录
                    print(f"❌ 文件 {fname} 没有后缀名,跳过下载")
                    continue

                fileId, fileItem = self.find_file_id_by_path(
                    fname, root_id=0, is_dir=False
                )
                if not fileId or not fileItem:
                    print(f"❌ 文件 {fname} 找不到")
                    continue

                download_res = self.file.download_info(fileId)
                download_url = download_res.downloadUrl
                if not download_url:
                    warnings.warn(f"无法获取 {fname} 的下载链接", stacklevel=2)
                    continue
                down_info = {
                    "url": download_url,
                    "filename": fileItem["filename"],
                    "md5": fileItem["etag"],
                }
                if onlyurl:
                    results.append(down_info)
                    continue
                else:
                    download_file(
                        url=down_info["url"],
                        output_path=down_info["filename"],
                        md5=down_info["md5"],
                        max_tries=5,  # Maximum number of retry attempts
                        retry_seconds=2,  # Initial retry delay in seconds
                    )
            except Exception as e:
                warnings.warn(f"下载 {fname} 时出错: {str(e)}", stacklevel=2)
                continue
        return results if onlyurl else None

    @retry(stop=stop_after_attempt(10), wait=wait_random(min=1, max=5))
    def _upload_file_data_common(
        self,
        get_upload_url_func: Callable,
        f: BinaryIO,
        preuploadID: int,
        start_seek: int,
        length: int,
        idx: int,
        task_upload_per: list,
    ) -> bool:
        """通用分片上传"""
        data_response = get_upload_url_func(preuploadID, idx + 1)
        presignedURL = data_response.data["presignedURL"]
        assert presignedURL, "获取 presignedURL 失败"
        f.seek(start_seek)
        requests.put(
            presignedURL,
            data=UploadInChunks(f, length, idx, task_upload_per, chunksize=1024),
            timeout=60,
        )
        task_upload_per[idx] = 100
        return True

    @validate_call
    def _upload_common(
        self,
        filename: Union[str, Path],
        upload_name: Union[str, Path, None, PurePosixPath],
        parentFileID: Union[int, str],
        overwrite: bool,
        use_oss: bool = False,
        duplicate: None | int = None,
        containDir: bool = False,
    ) -> int:
        """通用上传逻辑

        Args:
            filename (str | Path): 上传的文件名
            upload_name (str | Path | None): 上传云端的文件名. 如果为 None,则使用本地文件名.
            parentFileID (int | str): 上传到云端的目录 ID. 默认为根目录下, 如果使用use_oss,则为云端根目录为空, 如果不使用oss,则为云端目录为0
            overwrite (bool): 是否强制覆盖同名文件. 如果云端存在同名文件,则默认会报错.
            use_oss (bool ): 是否使用 OSS 上传. 默认为 False.
            duplicate (int | None): 当有相同文件名时,文件处理策略(1保留两者,新文件名将自动添加后缀,2覆盖原文件)
            containDir (bool): 上传文件是否包含路径,默认fasle

        Returns:
            文件 ID 或 -1

        """
        filename, upload_name = self._validate_and_prepare_paths(filename, upload_name)

        file_etag = self.file.calculate_md5(filename)
        file_size = Path(filename).stat().st_size
        uploader = self.oss if use_oss else self.file

        # OSS 不能列文件
        if not use_oss:
            if not isinstance(parentFileID, int):
                raise ValueError("parentFileID 必须为 int 类型")
            if not Path(upload_name).is_absolute():
                need_find_name = PurePosixPath("/") / str(upload_name).lstrip("./")
                need_find_name = str(need_find_name)

            file_id, file_item = self.find_file_id_by_path(
                need_find_name, root_id=parentFileID, is_dir=False
            )
            if file_id and overwrite:
                if isinstance(file_id, int):
                    file_id = [file_id]
                if not isinstance(file_id, list):
                    print("❌ fileId 不是list类型,强制删除失败, 退出")
                    return -1
                self.file.trash(file_id)
                warnings.warn(
                    f"云端文件 {need_find_name} 强制移除到回收站", stacklevel=2
                )

        with open(filename, "rb") as f:
            f.seek(0)
            create_kwargs = {
                "parentFileID": parentFileID,
                "filename": str(upload_name),
                "etag": file_etag,
                "size": file_size,
                "duplicate": duplicate,
                "containDir": containDir,
            }
            if use_oss:
                create_kwargs["type"] = 1
                create_kwargs.pop("duplicate", None)
                create_kwargs.pop("containDir", None)

            data_response = uploader.create(**create_kwargs)
            if data_response.code != 0:
                raise ValueError(data_response.message)

            if data_response.data is None:
                raise ValueError("上传接口返回数据为空,无法获取preuploadID等信息")

            if data_response.data is not None and data_response.data.get("reuse"):
                print("✅ 秒传成功....")
                return data_response.data["fileID"]

            preuploadID = data_response.data["preuploadID"]
            print("preuploadID:", preuploadID)
            sliceSize = data_response.data["sliceSize"]
            total_sliceNo = math.ceil(file_size / sliceSize)
            task_upload_per = [0.0] * total_sliceNo

            def upload_slice(sliceNo):
                start = sliceNo * sliceSize
                size = min(sliceSize, file_size - start)
                with open(filename, "rb") as f_slice:
                    f_slice.seek(start)
                    return self._upload_file_data_common(
                        uploader.get_upload_url,
                        f_slice,
                        preuploadID,
                        start,
                        size,
                        sliceNo,
                        task_upload_per,
                    )

            cpu_count = os.cpu_count() or 1
            max_workers = min(max(1, cpu_count - 1), total_sliceNo)
            print(" 开始上传到云端...")
            print(f" 文件被拆成 {total_sliceNo} 个分片, 分片大小: {sliceSize} 字节")
            print(f" 用 {max_workers} 个线程一起上传哦~")
            if max_workers == 1:
                for i in range(total_sliceNo):
                    if not upload_slice(i):
                        print(f"\n分片 {i} 上传失败,终止上传.")
                        return -1
                    avg = sum(task_upload_per) / total_sliceNo
                    print(f"\r进度: {avg:.1f}%(共{total_sliceNo}分片)", end="")
            else:
                with ThreadPoolExecutor(max_workers=max_workers) as executor:
                    future_to_slice = {
                        executor.submit(upload_slice, i): i
                        for i in range(total_sliceNo)
                    }
                    for future in as_completed(future_to_slice):
                        slice_id = future_to_slice[future]
                        if not future.result():
                            print(f"\n分片 {slice_id} 上传失败,终止上传.")
                            return -1
                        avg = sum(task_upload_per) / total_sliceNo
                        print(f"\r进度: {avg:.1f}%(共{total_sliceNo}分片)", end="")

            print("\n分片上传完成,开始合并分片...")
            complete_response = uploader.upload_complete(preuploadID)
            if complete_response.data is not None and complete_response.data.get(
                "completed"
            ):
                print("✅ 文件上传完成")
                return complete_response.data["fileID"]

            if complete_response.data is not None and complete_response.data.get(
                "async"
            ):
                while True:
                    time.sleep(2)
                    data_response = uploader.upload_async_result(preuploadID)
                    if data_response.data is not None and data_response.data.get(
                        "completed"
                    ):
                        print("✅ 文件上传完成")
                        return data_response.data["fileID"]
        return -1

    @validate_call
    def upload(
        self,
        filename: Union[str, Path],
        upload_name: Union[str, Path, None] = None,
        parentFileID: Union[int, str] = 0,
        overwrite: bool = False,
        duplicate: None | int = None,
        containDir: bool = False,
    ) -> int:
        """上传文件. 失败返回 -1,成功返回文件 ID. (只能处理文件,不能处理目录)

        Args:
            filename (str | Path): 上传的文件名
            upload_name (str | Path | None): 上传云端的文件名. 如果为 None,则使用本地文件名.
            parentFileID (int, str): 上传到云端的目录 ID. 默认为根目录下
            overwrite (bool): 是否强制覆盖同名文件. 如果云端存在同名文件,则默认会报错. 这个是先移除到回收站,再上传
            duplicate (int): 当有相同文件名时,文件处理策略(1保留两者,新文件名将自动添加后缀,2覆盖原文件)
            containDir (bool): 上传文件是否包含路径,默认 False.

        Returns:
            文件 ID 或 -1

        Example:
            ```python
            from cpan123 import Pan123openAPI
            pan123 = Pan123openAPI()
            # 上传文件
            fname = "xxxxx.zip"
            pan123.upload(fname, fname, 0, overwrite=True)
            print("上传完成")
            ```

        """
        # 排除 .开头的文件
        # if Path(filename).name.startswith("."):
        #     print(f"❌ 文件 {filename} 不能以 '.' 开头")
        #     return -1
        res = self._upload_common(
            filename, upload_name, parentFileID, overwrite, False, duplicate, containDir
        )
        return res

    @validate_call
    def upload_oss(
        self,
        filename: Union[str, Path],
        upload_name: Optional[Union[str, Path]] = None,
        parentFileID: Union[int, str] = "",
        overwrite: bool = False,
    ) -> int:
        """上传文件到oss(图床). 失败返回 -1,成功返回文件 ID.(建议根目录)

        Args:
            filename (str | Path): 上传的文件名
            upload_name (str | Path, optional): 上传云端的文件名. 如果为 None,则使用本地文件名.
            parentFileID (int, optional): 上传到云端的目录 ID. 默认为根目录下
            overwrite (bool, optional): 是否强制覆盖同名文件. 如果云端存在同名文件,则默认会报错.

        Returns:
            文件 ID 或 -1

        Example:
            ```python
            from cpan123 import Pan123openAPI
            pan123 = Pan123openAPI()
            # 上传文件到oss(图床)
            imgfile = "xxxx.png"
            res = pan123.upload_oss(imgfile, Path(imgfile).name)
            print(res)
            ```
        """
        res = self._upload_common(filename, upload_name, parentFileID, overwrite, True)
        return res

    @validate_call
    def find_file_id_by_path(
        self, path: str | PurePosixPath | Path, root_id: int = 0, is_dir: bool = True
    ) -> tuple[int, dict] | tuple[None, None]:
        """根据绝对路径查找对应文件(夹)的 fileId.

        该方法会递归查找路径中的每个部分,直到找到目标文件(夹)为止.

        - 如果路径中有部分不存在,则返回 None.
        - 注意:该方法仅支持绝对路径,且路径必须以 '/' 开头.
        - 例如:'/A/B/C' 是合法的绝对路径,而 'A/B/C' 不是.

        Args:
            path (str | PurePosixPath | Path): 云盘中的绝对路径,如 /A/B/C.
            root_id (int): 起始根目录 fileId, 默认从 0 开始.
            is_dir (bool): True 表示查找目录, False 表示查找文件.

        Returns:
            如果找到,返回包含 fileId 和文件信息字典的元组；如果未找到,返回 `(None, None)`.
        """
        path = PurePosixPath(path)
        if not path.is_absolute():
            raise ValueError("路径必须是绝对路径")
        if path.suffix and is_dir:
            raise ValueError("请求提供目录路径而不是文件路径")
        if not path.suffix and not is_dir:
            raise ValueError("请求提供文件路径而不是目录路径")

        parts = path.parts[1:]  # 忽略根 "/"
        current_id = root_id
        current_item = None
        for index, name in enumerate(parts):
            found = False
            last_file_id = None
            while True:
                res = self.file.list_v2(
                    parentFileId=current_id, lastFileId=last_file_id, limit=100
                )
                if not res.data or not res.data.get("fileList"):
                    print(f"❌ 在 {current_id} 下没有找到任何文件")
                    break

                for item in res.data["fileList"]:
                    if item["filename"] == name and item["trashed"] == 0:
                        # 中间部分必须是目录
                        if index < len(parts) - 1 and item["type"] != 1:
                            continue

                        # 最后一部分根据 is_dir 判断类型
                        if index == len(parts) - 1 and (
                            (is_dir and item["type"] != 1)
                            or (not is_dir and item["type"] != 0)
                        ):
                            continue
                        current_id = item["fileId"]
                        current_item = item
                        found = True
                        break

                if found:
                    break

                last_file_id = res.data.get("lastFileId", -1)
                if last_file_id == -1:
                    break

            if not found:
                return None, None
        if current_item is None:
            return None, None
        else:
            return current_item["fileId"], current_item

    @validate_call
    def _get_file_list(
        self, parentFileId: int, current_path: str = "", base_path: str = ""
    ) -> list[dict[str, Any]]:
        """
        递归获取指定 parentFileId 下的所有文件(包含子文件夹本身)

        Args:
            parentFileId (int): 当前目录的 fileId.
            current_path (str): 当前正在递归的完整路径(用于构造 full_path).
            base_path (str): 用户原始请求的基准路径(用于计算 relative_path).

        Returns:
            包含所有文件信息的字典列表,每项包含 full_path 和 relative_path 字段.
        """
        file_list = []
        lastFileId = None

        while True:
            res = self.file.list_v2(
                parentFileId=parentFileId, lastFileId=lastFileId, limit=100
            )

            if not res.data or not res.data.get("fileList"):
                break

            for item in res.data["fileList"]:
                item_path = f"{current_path}/{item['filename']}".rstrip("/")
                item["full_path"] = item_path

                # 计算相对路径(相对于 base_path)
                if base_path:
                    try:
                        relative_path = str(
                            PurePosixPath(item_path).relative_to(base_path)
                        )
                    except ValueError:
                        relative_path = item["filename"]
                else:
                    relative_path = item["filename"]

                item["relative_path"] = relative_path or "."

                # 如果是目录,则递归
                if item["type"] == 1:
                    file_list.append(item)  # 目录本身
                    file_list.extend(
                        self._get_file_list(item["fileId"], item_path, base_path)
                    )
                else:
                    file_list.append(item)

            lastFileId = res.data.get("lastFileId", -1)
            if lastFileId == -1:
                break

        # 检查每一项都是 dict,且含有 fileId 和 filename 字段,以及 full_path 和 relative_path 字段
        TypeAdapter(list[FileItem]).validate_python(file_list)
        return file_list

    @validate_call
    def list_files(self, dirnames: DirnamesType) -> list[dict] | None:
        """
        给定一个绝对目录路径,列出其下所有文件(含子目录)以及查询目录本身的文件信息.

        Args:
            dirnames (DirnamesType): 云盘中的绝对路径目录列表,必须以 '/' 开头或为整数 fileId.

        Returns:
            所有目录下的文件信息列表(包含 full_path 和 relative_path 字段).  如果没有找到目录,则返回 None.
        """
        all_files = []
        if not dirnames:
            print("❌ 目录名不能为空")
            return None

        if isinstance(dirnames, (str, int, Path, PurePosixPath)):
            dirnames = [dirnames]
        if isinstance(dirnames, list) and not all(
            isinstance(dirname, (str, int, Path, PurePosixPath)) for dirname in dirnames
        ):
            raise ValueError("dirnames 必须是字符串或 PurePosixPath 的列表")

        for dirname in dirnames:
            if isinstance(dirname, (str, PurePosixPath, Path)):
                dirname = PurePosixPath(dirname)
                # 校验
                if not dirname.is_absolute():
                    print(f"❌ 跳过非法路径: {dirname}(必须是绝对路径)")
                    continue
                if dirname.suffix:
                    print(f"❌ 跳过文件路径: {dirname}(必须是目录)")
                    continue
                if dirname == PurePosixPath("/"):
                    print("⚠️ 跳过根目录, 请指定更具体的文件夹)")
                    continue
                # 获取 fileId 并递归查找文件
                file_id, file_item = self.find_file_id_by_path(dirname)
                if file_id is None:
                    print("❌ 没有找到该目录")
                    return None
            elif isinstance(dirname, int):
                # 如果是整数,直接使用
                file_id = dirname

            files = self._get_file_list(
                file_id, current_path=str(dirname), base_path=str(dirname)
            )
            if files is None:
                continue
            # 获取目录本身的 id
            thisdir = self.file.infos([file_id])
            if (
                thisdir.data
                and thisdir.data.get("fileList")
                and len(thisdir.data["fileList"]) == 1
            ):
                data = thisdir.data["fileList"][0]
                data["full_path"] = str(dirname)
                data["relative_path"] = "."
                all_files.append(data)
            all_files.extend(files)

        # 检查每一项都是 dict,且含有 fileId 和 filename 字段,以及 full_path 和 relative_path 字段
        TypeAdapter(list[FileItem]).validate_python(all_files)
        return all_files

    @validate_call
    def _download_single_file(self, file: dict, output_path: str | Path = ".") -> None:
        """
        下载单个文件到指定路径.

        Args:
            file (dict): 文件信息字典,必须包含 'fileId' 和 'etag' 字段.
            output_path (str | Path): 下载保存的目录,默认当前目录.

        Returns:
            None
        """
        # 过滤掉回收站的文件
        if file["trashed"] == 1:
            return
        # 过滤掉目录
        if file["type"] == 1:
            # temp_path = Path(output_path) / file["relative_path"]
            # Path(temp_path).mkdir(parents=True, exist_ok=True)
            return
        download_url = self.file.download_info(file["fileId"]).downloadUrl
        if not download_url:
            print(f"❌ 无法获取下载链接: {file['relative_path']}")
            return
        local_path = Path(output_path) / file["relative_path"]
        Path(local_path).parent.mkdir(parents=True, exist_ok=True)
        download_file(
            url=download_url,
            output_path=str(local_path),
            md5=file["etag"],
            overwrite=True,
            max_tries=5,
            retry_seconds=2,
        )

    @validate_call
    def download_dir(
        self,
        dirnames: DirnamesType,
        output_path: str | Path = ".",
    ) -> None:
        """
        下载指定目录下的所有文件(含子目录).

        Args:
            dirname (DirnamesType): 云盘中的绝对路径目录列表,必须以 '/' 开头或为整数 fileId 的列表.
            output_path (str | Path): 下载保存的目录,默认当前目录.

        Returns:
            None
        """
        all_files = self.list_files(dirnames)
        if all_files is None:
            print("❌ 没有找到该目录")
            return

        # 获取下载链接,下载文件
        Path(output_path).mkdir(parents=True, exist_ok=True)
        print(f" 开始下载到 {output_path}...")
        for file in all_files:
            # 过滤掉查询目录本身
            if str(dirnames) in [str(file["fileId"]), file["filename"]]:
                continue
            self._download_single_file(file, output_path)

    @validate_call
    def upload_dir(
        self,
        dirname: Union[str, Path],
        parentFileID: int = 0,
        upload_dirname: str | None = None,
    ) -> None:
        """
        上传目录到云端.

        Args:
            dirname (Union[str, Path]): 本地目录路径.
            parentFileID (int): 云端目标父目录的 ID,默认为根目录为 0
            upload_dirname (Optional[str]): 云端新建目录的名称. 如果为 None,则使用 parentFileID 指定的目录.

        Returns:
            None
        """
        local_dir = Path(dirname)
        if not local_dir.exists() or not local_dir.is_dir():
            raise ValueError(f"指定的目录不存在或不是目录: {local_dir}")
        # 不能是.开头的文件
        # if Path(dirname).name.startswith("."):
        #     print(f"❌ 目录 {dirname} 不能以 '.' 开头")
        #     return

        # 创建云端目录(如果指定了 upload_dirname)
        target_dir_id = parentFileID
        if upload_dirname:
            res = self.file.mkdir(name=upload_dirname, parentID=parentFileID, skip=True)
            if not res or res.data is None or res.code == 1:
                print(f"❌ 创建目录失败: {upload_dirname}")
                return
            target_dir_id = res.data["dirID"]

        # 遍历本地目录并上传文件
        files = [f for f in local_dir.rglob("*") if f.is_file()]
        print(f"正在上传 {len(files)} 个文件...")
        for file_path in files:
            try:
                print(f"正在上传: {file_path}")
                res = self.upload(
                    str(file_path), str(file_path), target_dir_id, False, 1, True
                )
                if res == -1:
                    print(f"❌ 上传失败: {file_path}")
                else:
                    print(f"✅ 上传成功: {file_path} -> {res}")
            except Exception as e:
                print(f"❌ 上传过程中出现异常: {file_path},错误信息: {e}")

    @validate_call
    def delete_dir(
        self,
        dirnames: DirnamesType,
    ) -> None:
        """
        删除指定目录及其下的所有文件(含子目录).

        Args:
            dirnames (DirnamesType): 云盘中的绝对路径目录列表,必须以 '/' 开头或为整数 fileId.

        Returns:
            None
        """
        all_files = self.list_files(dirnames)
        if all_files is None:
            print("❌ 没有找到该目录")
            return
        if len(all_files) == 0:
            print("❌ 目录下没有文件")
            return
        all_files_id = [file["fileId"] for file in all_files if file["trashed"] == 0]
        if not all_files_id:
            print("❌ 目录下没有文件")
            return
        ######  好像不能批量删除,只能单个删除
        ## 单个的删除
        [self.file.trash([file_id]) for file_id in all_files_id]
        print("✅ 删除完成")

        ##### 批量删除
        # n = len(all_files_id)
        # for i in range(0, n, 100):
        #     if i + 100 > n:
        #         batch_ids = all_files_id[i:]
        #     else:
        #         batch_ids = all_files_id[i : i + 100]
        #     res = self.file.trash(batch_ids)
        #     if res.code == 0:
        #         print(f"✅ 删除成功: {batch_ids}")
        #     else:
        #         print(f"❌ 删除失败: {batch_ids},错误信息: {res.message}")


if __name__ == "__main__":
    pan123 = Pan123openAPI()
    # # 下载单个文件
    fname = "word3prefect6300data.rar"
    pan123.download(fname, onlyurl=False)

    # # # 上传文件
    # pan123.upload(fname, fname, 0, overwrite=True)
    # print("上传完成")



from pathlib import PurePosixPath

from cpanbd.file import File  # type: ignore
from cpanbd.utils.md5 import decrypt_md5  # type: ignore
from pydantic import validate_call

from .pan import Pan123openAPI


@validate_call
def baiduTo123(
    filebd: str,
    file123: str,
) -> bool:
    """
    百度网盘秒传到123网盘(前提是两边都有这个文件), 秒传成功返回 True, 否则返回 False

    百度网盘服务器上有这个文件, 123网盘服务器上也有这个文件, 才能秒传, 否则不上传到123网盘

    !!! note "注意"
        依赖 `cpanbd` 库, 请先安装 `pip install cpanbd`


    Args:
        filebd: 百度网盘文件路径 (绝对路径), 以 / 开头,只能是一个件
        file123: 115网盘文件路径 (绝对路径), 以 / 开头, 只能是一个文件

    Example:
        ```python
        from cpanbd import baiduTo123
        # 百度网盘文件路径
        filebd = "/BaiduNetdiskDownload/book/pdf/xxx.pdf"
        # 123网盘文件路径
        file123 = "/book/pdf/xxxx.pdf"
        baiduTo123(filebd, file123)
        ```

    """
    assert filebd.startswith("/"), "❌ 百度网盘文件路径必须以 / 开头"
    assert file123.startswith("/"), "❌ 123网盘文件路径必须以 / 开头"
    # 判断是文件而不是文件夹
    assert PurePosixPath(filebd).suffix != "", "❌ 百度网盘文件路径必须是文件"
    assert PurePosixPath(file123).suffix != "", "❌ 123网盘文件路径必须是文件"
    bddir = str(PurePosixPath(filebd).parent)
    bdname = str(PurePosixPath(filebd).name)

    # p123dir = str(PurePosixPath(file123).parent)
    # p123name = str(PurePosixPath(file123).name)
    file = File()
    pan123 = Pan123openAPI()
    listFiles = file.search(key=bdname, dir=bddir, web=0)
    if not listFiles or "list" not in listFiles:
        print("❌ 百度网盘搜索文件失败")
        return False

    fileinfo = None
    for item in listFiles["list"]:
        if item["path"] == filebd and item["isdir"] == 0:
            fileinfo = item
            break
    if not fileinfo:
        print("❌ 百度网盘文件不存在")
        return False
    filesize = fileinfo["size"]
    filemd5 = fileinfo["md5"]

    ### 上传到123网盘
    res = pan123.file.create(
        parentFileID=0,
        filename=file123,
        etag=decrypt_md5(filemd5),
        size=filesize,
        duplicate=1,  # 1 表示保留两者
        containDir=True,  # 是否包含目录
    )
    if res.data and res.data["reuse"] is True:
        print("✅ 成功秒传到123网盘")
        return True
    else:
        print("❌ 秒传失败")
        return False


if __name__ == "__main__":
    # 百度网盘文件路径
    filebd = "/BaiduNetdiskDownload/book/pdf/[伍德里奇]横截面与面板数据的经济计量分析(中文版).pdf"
    # 123网盘文件路径
    file123 = "/book/pdf/[伍德里奇]横截面与面板数据的经济计量分析(中文版).pdf"
    baiduTo123(filebd, file123)



# tests/test_core.py

import os

import pytest

from cpan123.utils.core import FieldParser


def test_constant_int_parsing():
    field = FieldParser.parse("retries", 3)
    assert field.default == 3
    assert field.type_ is int
    assert field.is_constant


def test_simple_template_parsing(monkeypatch):
    monkeypatch.setenv("API_KEY", "secret123")
    field = FieldParser.parse("api_key", "{{ API_KEY }}")
    assert field.is_template
    assert field.default == "secret123"
    assert field.template_key == "API_KEY"


def test_embedded_template_resolution():
    context = {"user": {"id": "u123"}}
    template = "data/{{{ user.id }}}/result"
    resolved = FieldParser._replace_embedded_templates(template, context)
    assert resolved == "data/u123/result"


def test_default_and_type_parsing():
    field = FieldParser.parse("enabled", "true: bool: required")
    assert field.required
    assert field.default is True
    assert field.type_ is bool


def test_field_with_type_only():
    field = FieldParser.parse("foo", "int: optional")
    assert field.default == 0
    assert field.type_ is int


def test_minimal_required_input_success():
    schema = {"method": "GET: str: required", "timeout": "3: int: optional"}
    parsed = FieldParser.parse_dict(schema)
    user_input = {"method": "POST"}
    result = FieldParser.validate_and_fill_input(parsed, user_input)
    assert result["method"] == "POST"
    assert result["timeout"] == 3


def test_missing_required_field_raises():
    schema = {"url": "str: required", "method": "GET: str: optional"}
    parsed = FieldParser.parse_dict(schema)
    with pytest.raises(ValueError, match=r"缺少必填字段: `url`"):
        FieldParser.validate_and_fill_input(parsed, {})


def test_nested_structure_with_required_field():
    schema = {"config": {"debug": "false: bool: optional", "host": "str: required"}}
    parsed = FieldParser.parse_dict(schema)
    result = FieldParser.validate_and_fill_input(parsed, {"host": "localhost"})
    assert result["config"]["host"] == "localhost"
    assert result["config"]["debug"] is False


def test_non_string_value_in_string_format_field():
    field = FieldParser.parse("count", "5: int: required")
    assert field.default == 5
    assert field.type_ is int


def test_type_cast_failure_fallbacks_to_str():
    # Deliberate type mismatch: "abc" to int should fallback
    field = FieldParser.parse("bad_int", "abc: int: optional")
    assert field.default == "abc"
    assert field.type_ is int  # Still int, but fallback str as value


def test_type_keyword_only_constant_field():
    field = FieldParser.parse("is_active", "bool")
    assert field.is_constant
    assert field.default == "bool"


def test_null_value_yields_none():
    field = FieldParser.parse("token", "null")
    assert field.default is None
    assert field.type_ is type(None)


def test_skip_keys_in_parse_dict():
    data = {
        "method": "GET: str: optional",
        "comment": "not parsed",
        "schema_": {},
        "files": [],
    }
    parsed = FieldParser.parse_dict(data)
    assert "method" in parsed
    assert "comment" not in parsed
    assert "schema_" not in parsed
    assert "files" not in parsed


def test_template_parsing():
    os.environ["MY_ENV_VAR"] = "12345"

    schema = {
        "api_key": "{{ MY_ENV_VAR }}",
        "timeout": "5: int: optional",
        "config": {
            "path": "default/path: str: optional",
            "enabled": "true: bool: required",
        },
    }

    parsed = FieldParser.parse_dict(schema)
    filled = FieldParser.validate_and_fill_input(parsed, {"enabled": True})

    assert filled["api_key"] == "12345"
    assert filled["timeout"] == 5
    assert filled["config"]["path"] == "default/path"
    assert filled["config"]["enabled"] is True


def test_embedded_template_parsing():
    template = {"url": "https://example.com/{{{ user.name }}}/info"}
    user_input = {"user": {"name": "Alice"}}
    parsed = FieldParser.parse_dict(template)
    filled = FieldParser.validate_and_fill_input(parsed, user_input)

    assert filled["url"] == "https://example.com/Alice/info"


def test_files():
    with open("图吧工具箱2409安装程序.exe", "rb") as f:
        chunk = f.read(1024 * 1024 * 10)

        files = [("file", ("part", chunk))]

        res = {
            "url": "/rest/2.0/pcs/superfile2",
            "path": "xxx",
            "method": "POST",
            "partseq": 1,
            "files": files,
        }
        parsed = FieldParser.parse_dict(res)
        filled = FieldParser.validate_and_fill_input(parsed, {"files": files})
        assert filled["url"] == "/rest/2.0/pcs/superfile2"
        assert filled["files"] == files, "文件上传失败"



from __future__ import annotations

import sys
from pathlib import Path
from typing import Annotated, Any, Optional

from pydantic import Field, conlist

from .utils.api import Auth
from .utils.baseapiclient import BaseApiClient, auto_args_call_api
from .utils.checkdata import DataResponse
from .utils.md5 import calculate_md5


class File(BaseApiClient):
    def __init__(self, auth: Optional[Auth] = None) -> None:
        super().__init__(filepath="file", auth=auth)

    @auto_args_call_api("create")
    def create(
        self,
        parentFileID: int,
        filename: str,
        etag: str = Field(default="", max_length=32),
        size: int = Field(default=0, gt=0),
        duplicate: int = 1,
        containDir: bool = False,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """
        创建文件上传任务.

        - 文件名不能全部是空格
        - 开发者上传单文件大小限制10GB

        Args:
            parentFileID (int): 父目录id,上传到根目录时填写 0
            filename (str): 文件名要小于255个字符且不能包含一些特殊字符(不建议重名)
            etag (str): 文件的md5值, 如果不传入,则自动计算
            size (int): 文件大小, 单位字节
            duplicate (int): 当有相同文件名时,文件处理策略(1保留两者,新文件名将自动添加后缀,2覆盖原文件)
            containDir (bool):上传文件是否包含路径,默认fasle
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def get_upload_url(
        self,
        preuploadID: str,
        sliceNo: int = Field(ge=1),
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取文件上传地址.

        Args:
            preuploadID (str): 预上传ID
            sliceNo (int): 分片序号,从1开始自增
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def list_upload_parts(
        self,
        preuploadID: str,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取文件上传分片列表.

        Args:
            preuploadID (str): 预备上传ID
            skip (bool): 是否跳过响应数据的模式校验
        """
        # 获取 API

    @auto_args_call_api()
    def upload_complete(
        self,
        preuploadID: str,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """文件上传完成后请求

        Args:
            preuploadID (str): 预上传ID
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def upload_async_result(
        self,
        preuploadID: str,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """异步轮询获取上传结果

        Args:
            preuploadID (str): 预上传ID
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def mkdir(
        self,
        name: str,
        parentID: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """创建目录
        Args:
            name (str): 目录名称
            parentID (int): 父目录ID
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def name(
        self,
        fileId: int,
        fileName: str,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """单个文件重命名

        Args:
            fileId (int): 文件ID
            fileName (str): 新文件名
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def rename(
        self,
        renameList: list[str],
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """批量重命名文件,最多支持同时30个文件重命名

        Args:
            renameList (list): 重命名列表,格式为 [14705301|测试文件重命名","14705306|测试文件重命名.mp4"]
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def trash(
        self,
        fileIDs: Annotated[list[int], conlist(int, max_length=100)],
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """删除的文件,会放入回收站中

        Args:
            fileIDs (list[int]): 文件id数组,一次性最大不能超过 100 个文件
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def recover(
        self,
        fileIDs: Annotated[list[int], conlist(int, max_length=100)],
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """将回收站的文件恢复至删除前的位置

        Args:
            fileIDs (list): 文件id数组,一次性最大不能超过 100 个文件
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def delete(
        self,
        fileIDs: Annotated[list[int], conlist(int, max_length=100)],
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """彻底删除文件前,文件必须要在回收站中,否则无法删除

        Args:
            fileIDs (list): 文件id数组,参数长度最大不超过 100
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def detail(
        self,
        fileID: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取单个文件详情

        Args:
            fileID (int): 文件ID
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def infos(
        self,
        fileIds: list[int],
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取多个文件详情

        Args:
            fileIds (list): 文件ID列表
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api("list_v2")
    def list_v2(
        self,
        parentFileId: int,
        limit: int = 100,
        searchData: Optional[str] = None,
        searchMode: Optional[int] = 0,
        lastFileId: Optional[int] = None,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取文件列表(V2版本, v1弃用)

        Args:
            parentFileId (int): 文件夹ID,根目录传 0
            limit (int): 每页文件数量,最大不超过100
            searchData (str, optional):搜索关键字将无视文件夹ID参数. 将会进行全局查找
            searchMode (int, optional): 搜索模式,0:模糊搜索,1:精确搜索,默认为0
            lastFileId (int, optional): 翻页查询时需要填写
            skip (bool): 是否跳过响应数据的模式校验
        """
        # 如果要对参数进行校验,需要 Field 等参数校验方法
        # 默认已开启函数参数校验
        # 也可以在函数体中进行参数校验了, 不能参与计算
        # print(self.list_v2.__name__)
        # print(self.list_v2.__doc__)
        if limit > 100 and limit < 0:
            print("limit 参数最大值为 100,请修改后重试")
            sys.exit(1)

    @auto_args_call_api("list_v1")
    def list_v1(
        self,
        parentFileId: int = 0,
        page: int = 1,
        limit: int = Field(default=100, gt=0, le=100),
        orderBy: str = "file_id",
        orderDirection: str = "asc",
        trashed: Optional[bool] = True,
        searchData: Optional[str] = "",
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        pass

    def list(self, *args: Any, **kwargs: Any) -> DataResponse:
        """获取文件列表(V1版本, v2弃用)

        具体参考 list_v2的用法,这个只是一个别名, 因为list和python中的list冲突了(为了尽力保证和url的末端一致性)

        Args:
            args (Any): 传入参数
            kwargs (Any): 传入参数
        """
        # 获取 API
        return self.list_v2(*args, **kwargs)

    @auto_args_call_api()
    def move(
        self,
        fileIDs: list[int],
        toParentFileID: int = 0,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """批量移动文件,单级最多支持100个

        Args:
            fileIDs (list[int]): 文件id数组
            toParentFileID (int): 要移动到的目标文件夹id,移动到根目录时填写 0
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def download_info(
        self,
        fileId: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取文件下载地址

        Args:
            fileId (int): 文件ID
            skip (bool): 是否跳过响应数据的模式校验
        """
        # 获取 API
        # return self._call_api("download_info", fileID=fileID)

    def calculate_md5(self, file_path: Path | str) -> str:
        """计算文件的 MD5 值

        Args:
            file_path (Path | str): 文件路径,可以是 Path 对象或字符串

        Returns:
            str: 文件的 MD5 值

        Raises:
            AssertionError: 文件不存在或路径不是文件或文件大小为0

        """
        return calculate_md5(file_path)


if __name__ == "__main__":
    file = File()

    print(file.list_v2(0, 10, "data").data)



import os
import re
from typing import Any, Dict, Optional, Type, Union

from pydantic import ConfigDict
from pydantic.dataclasses import dataclass

from .const import (
    DEFAULT_BY_TYPE,
    EMBEDDED_TEMPLATE_PATTERN,
    TEMPLATE_PATTERN,
    TYPE_MAP,
)


def parse_bool(value: str) -> bool:
    return value.strip().lower() in ("true")


@dataclass
class ParsedField:
    name: str
    type_: Type
    required: bool
    default: Any
    is_template: bool = False
    template_key: Optional[str] = None
    is_constant: bool = False

    model_config = ConfigDict(extra="forbid")


class FieldParser:
    SKIP_KEYS = {"comment", "schema_", "files", "headers", "auth", "response_schema"}

    @staticmethod
    def parse(name: str, raw_value: Any) -> ParsedField:
        if raw_value is None or (
            isinstance(raw_value, str)
            and any(x in raw_value.lower() for x in ("none", "null"))
        ):
            return ParsedField(
                name=name, type_=type(None), required=False, default=None
            )

        if not isinstance(raw_value, str):
            return ParsedField(
                name=name,
                type_=type(raw_value),
                required=False,
                default=raw_value,
                is_constant=True,
            )

        val = raw_value.strip()

        # {{ KEY }} 整体模板（环境变量）
        match = TEMPLATE_PATTERN.fullmatch(val)
        if match:
            return ParsedField(
                name=name,
                type_=str,
                required=False,
                default=os.getenv(match.group(1), f"{{{{ {match.group(1)} }}}}"),
                is_template=True,
                template_key=match.group(1),
            )

        # 嵌入式模板 {{{ key }}}
        if EMBEDDED_TEMPLATE_PATTERN.search(val):
            return ParsedField(
                name=name,
                type_=str,
                required=False,
                default=val,
                is_template=True,
                template_key=None,
            )

        # "default: type: required"
        parts = [p.strip() for p in val.split(": ")]
        if len(parts) == 3:
            default_str, type_str, required_str = parts

            required = required_str.lower() == "required"
            py_type_raw = TYPE_MAP.get(type_str.lower(), str)
            py_type = py_type_raw if isinstance(py_type_raw, type) else str

            default_value = default_str
            if default_str is not None:
                try:
                    if py_type is bool:
                        default_value = parse_bool(default_str)
                    else:
                        default_value = py_type(default_str)
                except Exception:
                    default_value = default_str
            elif required:
                default_value = None

            return ParsedField(
                name=name,
                type_=py_type,
                required=required,
                default=default_value,
            )

        elif len(parts) == 2:
            type_str, required_str = parts
            try:
                default_str = DEFAULT_BY_TYPE.get(type_str.lower(), "")
                type_ = TYPE_MAP.get(type_str.lower(), str)
                required = required_str.lower() == "required"
                return ParsedField(
                    name=name,
                    type_=type_,
                    required=required,
                    default=default_str,
                )
            except Exception as e:
                raise ValueError(f"无法解析字段: {name}, 值: {val}") from e

        elif len(parts) == 1:
            # 只有一个值，默认是字符串类型, 且不是必填, 认为是常量
            return ParsedField(
                name=name,
                type_=str,
                required=False,
                default=val,
                is_constant=True,
            )

        else:
            raise ValueError(f"无法解析字段: {name}, 值: {val}")

    @staticmethod
    def _replace_embedded_templates(value: str, context: Dict[str, Any]) -> str:
        def _resolve_key_path(context: Dict[str, Any], key_path: str) -> Any:
            keys = key_path.split(".")
            val = context
            for k in keys:
                if isinstance(val, dict) and k in val:
                    val = val[k]
                else:
                    return f"{{{{{{ {key_path} }}}}}}"  # fallback 原样
            return val

        def replace_match(match: re.Match) -> str:
            key_path = match.group(1)
            return str(_resolve_key_path(context, key_path))

        return EMBEDDED_TEMPLATE_PATTERN.sub(replace_match, value)

    @staticmethod
    def parse_dict(data: Dict[str, Any]) -> Dict[str, Any]:
        parsed = {}
        for key, value in data.items():
            if key in FieldParser.SKIP_KEYS:
                continue

            if isinstance(value, dict):
                parsed[key] = FieldParser.parse_dict(value)
            else:
                parsed[key] = FieldParser.parse(key, value)

        return parsed

    @staticmethod
    def _resolve_template_value(field: ParsedField, context: Dict[str, Any]) -> Any:
        if field.is_template and isinstance(field.default, str):
            return FieldParser._replace_embedded_templates(field.default, context)
        return field.default

    @staticmethod
    def validate_and_fill_input(
        template: Dict[str, Union[ParsedField, dict]],
        user_input: Dict[str, Any],
        path: str = "",
    ) -> Dict[str, Any]:
        result = {}
        ## 添加特殊字段
        template["skip"] = ParsedField(
            name="skip",
            type_=bool,
            required=False,
            default=False,
            is_constant=True,
        )

        for key, field_or_nested in template.items():
            full_key = f"{path}.{key}" if path else key

            if isinstance(field_or_nested, dict):
                # 处理嵌套结构
                nested_result = {}
                for nested_key, nested_field in field_or_nested.items():
                    nested_full_key = f"{full_key}.{nested_key}"

                    if isinstance(nested_field, dict):
                        raise ValueError(f"暂不支持三层嵌套字段: {nested_full_key}")

                    if nested_key in user_input:
                        # 保留 None 值
                        nested_result[nested_key] = user_input[nested_key]
                    elif nested_field.required:
                        raise ValueError(f"缺少必填字段: `{nested_full_key}`")
                    else:
                        nested_result[nested_key] = FieldParser._resolve_template_value(
                            nested_field, user_input
                        )

                result[key] = nested_result

            else:
                # 顶级字段处理
                if key in user_input:
                    result[key] = user_input[key]
                elif field_or_nested.required:
                    raise ValueError(f"缺少必填字段: `{full_key}`")
                else:
                    result[key] = FieldParser._resolve_template_value(
                        field_or_nested, user_input
                    )

        # 特殊保留字段：直接覆盖模板中的默认值
        for special in FieldParser.SKIP_KEYS:
            if special in ["comment"]:
                continue
            if special in user_input:
                result[special] = user_input[special]

        return result



from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()


def test_directlink():
    # 启用直连空间, 传入的是文件夹id
    # fileinfo = pan123.file.list_v2(0, 50)
    # print("----" * 10)
    dirid = 22139321
    fileinfo = pan123.file.detail(fileID=dirid)
    print(f"获取文件信息: \n{fileinfo.data}")
    assert fileinfo.data, "获取文件信息失败"
    assert fileinfo.data["type"] == 1, "不是文件夹类型"
    # 获取文件夹里面的文件列表
    file_list = pan123.file.list_v2(dirid, 100)

    print("----" * 10)
    print(f"获取文件列表: \n{file_list.data}")

    res = pan123.directlink.enable(fileID=dirid)
    print("----" * 10)
    print(f"启用直连空间: \n{res.data}")

    # 获取直链链接( 这里是直连空间下的文件id)
    res = pan123.directlink.url(fileID=22139325)
    print("----" * 10)
    print(f"获取直链链接: \n{res.data}")

    # 禁用直连空间
    res = pan123.directlink.disable(fileID=dirid)
    print("----" * 10)
    print(f"禁用直连空间: \n{res.data}")



from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()

share_baseurl = "https://www.123pan.com/s/"  # 分享链接的基础URL


def test_share():
    ## 先存一份数据到本地
    # file_list = pan123.file.list_v2(0, 10, "国科")
    # print(file_list.data)
    share = pan123.share.create(
        shareName="测试分享", shareExpire=1, fileIDList="11890750,12002051", skip=True
    )

    # 分享码,请将分享码拼接至 https://www.123pan.com/s/ 后面访问,即是分享页面
    print("----" * 10)
    assert share.data, "创建分享链接失败"
    print(share.data)
    print("分享链接:", share_baseurl + share.data["shareKey"])

    share2 = pan123.share.create(
        shareName="测试分享2", shareExpire=1, fileIDList="12004728,12004738", skip=True
    )

    print("----" * 10)
    assert share2.data, "创建分享链接失败"
    print(share2.data)
    print("分享链接2:", share_baseurl + share2.data["shareKey"])

    share3 = pan123.share.info(
        shareIdList=[share2.data["shareID"]],
        trafficSwitch=1,
        trafficLimitSwitch=1,
        trafficLimit=1000000,
    )
    print("----" * 10)
    print(f"修改分享链接: \n{share3.data}")

    # 获取分享链接信息
    res = pan123.share.list(limit=10, lastShareId=0)
    print("----" * 10)
    print(f"获取分享链接: \n{res.data}")


if __name__ == "__main__":
    test_share()



from typing import BinaryIO, Dict, List, Literal, Optional

import requests
from pydantic import BaseModel, ConfigDict, field_validator


class JsonInput(BaseModel):
    method: str
    url: str
    data: Optional[Dict] = None
    params: Optional[Dict] = None
    schema_: Optional[Dict] = None
    comment: Optional[str] = None
    response_schema: Optional[Dict] = None

    model_config = ConfigDict(extra="forbid")  # 禁止额外字段

    @field_validator("method")
    @classmethod
    def validate_method(cls, v: str) -> str:
        allowed = {"get", "post", "put", "delete"}
        v_lower = v.lower()
        if v_lower not in allowed:
            raise ValueError(f"method 必须是以下之一: {allowed}")
        return v_lower  # 统一返回小写形式


class BaseResponse(BaseModel):
    """
    基础响应类.
    """

    code: Literal[0]
    message: str = ""
    data: Optional[dict] = None
    x_traceID: Optional[str] = None


class DataResponse:
    response_data = None

    def __init__(self, response=None):
        """
        服务器返回数据类.

        :param response: 服务器返回原 请求 数据
        :param code: 自定义响应值.
        :param message: 自定义消息.
        :param data: 自定义数据.
        """
        if response is None:
            self.response = None
            self.response_data = {
                "code": -1,
                "message": "",
                "data": "",
                "x-traceID": "",
            }
            return
        elif isinstance(response, dict):
            self.response = None
            self.response_data = response
        elif isinstance(response, requests.Response):
            self.response = response
            try:
                self.response_data = response.json()
            except ValueError:
                self.response_data = response.text

    @property
    def data(self):
        """
        服务器返回数据段.
        :return: str
        """
        if isinstance(self.response_data, dict):
            return self.response_data.get("data")
        return None

    @property
    def x_traceID(self):
        """
        服务器记录值.
        :return: str
        """
        if isinstance(self.response_data, dict):
            return self.response_data.get("x-traceID")
        return None

    @property
    def message(self):
        """
        服务器返回消息.
        :return: str
        """
        if isinstance(self.response_data, dict):
            return self.response_data.get("message")
        return None

    @property
    def success(self):
        """
        是否成功
        :return: bool
        """
        if isinstance(self.response_data, dict):
            return self.response_data.get("code") == 0
        return False

    @property
    def code(self):
        if isinstance(self.response_data, dict):
            return self.response_data.get("code")
        return None

    @property
    def fileList(self):
        """
        服务器返回文件列表.
        :return: list
        """
        if isinstance(self.response_data, dict):
            data = self.response_data.get("data", None)
            if isinstance(data, dict):
                return data.get("fileList", None)
        return None

    @property
    def downloadUrl(self):
        """
        服务器返回下载链接.
        :return: str
        """
        if isinstance(self.response_data, dict):
            return self.response_data.get("data", {}).get("downloadUrl", None)
        return None

    def __repr__(self):
        """
        返回响应的字符串表示.
        :return: str
        """
        return (
            f"DataResponse(code={self.code}, message={self.message}, data={self.data})"
        )

    def __str__(self):
        """
        返回响应的字符串表示.
        :return: str
        """
        return (
            f"DataResponse(code={self.code}, message={self.message}, data={self.data})"
        )


class UploadInChunks:
    def __init__(
        self,
        fp: BinaryIO,
        length: int,
        idx: int,
        progress_ref: List[float],
        chunksize: int = 1 << 13,
    ) -> None:
        """
        初始化一个支持分块上传的文件读取器.

        Args:
            fp (BinaryIO): 文件对象(必须是以二进制方式打开的).
            length (int): 文件总大小(字节数).
            idx (int): 当前任务的索引,用于在 progress_ref 中更新进度.
            progress_ref (List[float]): 外部进度列表的引用.
            chunksize (int, optional): 每次读取的字节大小,默认 8192(8KB)
        """
        self.fp = fp
        self.idx = idx
        self.chunksize = chunksize
        self.totalsize = length
        self.readsofar = 0
        self.progress_ref = progress_ref

    def __iter__(self) -> "UploadInChunks":
        """
        实现迭代器接口.

        Returns:
            UploadInChunks: 迭代器本身.
        """
        return self

    def __next__(self) -> bytes:
        """
        迭代读取下一个数据块.

        Returns:
            bytes: 下一块文件数据.

        Raises:
            StopIteration: 当读取完毕时抛出.
        """
        if self.readsofar >= self.totalsize:
            raise StopIteration

        # 计算应该读取的大小
        read_len = min(self.chunksize, self.totalsize - self.readsofar)
        data: bytes = self.fp.read(read_len)

        if not data:
            raise StopIteration

        self.readsofar += len(data)
        self.progress_ref[self.idx] = self.readsofar * 100 / self.totalsize
        return data

    def __len__(self) -> int:
        """
        返回总字节数.

        Returns:
            int: 文件总大小.
        """
        return self.totalsize


if __name__ == "__main__":
    import json5

    with open("src/cpan123/apijson/file.json", "r", encoding="utf-8") as f:
        data = json5.load(f)

    for name, conf in data.items():
        try:
            validated = JsonInput.model_validate(conf)
            print(f"{name} ✅ 验证通过: {validated}")
        except Exception as e:
            print(f"{name} ❌ 验证失败: {e}")



import sys
from pathlib import Path
from typing import Dict

from pydantic import TypeAdapter

from cpan123.utils.checkdata import JsonInput


def test_json():
    import json5

    jsondir = Path("src/cpan123/apijson")
    jsonfiles = [f for f in jsondir.glob("*.json") if f.is_file()]
    for path in jsonfiles:
        with open(path, "r", encoding="utf-8") as file:
            try:
                data: dict = json5.load(file)
            except Exception as e:
                print(f"❌ JSON 解析失败: {path}\n错误: {e}")
                sys.exit(1)
        user_list_adapter = TypeAdapter(Dict[str, JsonInput])

        user_list_adapter.validate_python(data)



from __future__ import annotations

from typing import Optional

from .utils.api import Auth
from .utils.baseapiclient import BaseApiClient, auto_args_call_api
from .utils.checkdata import DataResponse


class DirectLink(BaseApiClient):
    def __init__(self, auth: Optional[Auth] = None) -> None:
        super().__init__(filepath="directlink", auth=auth)

    @auto_args_call_api()
    def enable(
        self,
        fileID: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """启用直链

        Args:
            fileID (int): 启用直链空间的文件夹的fileID
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api("url")
    def url(
        self,
        fileID: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取直链 URL

        Args:
            fileID (int): 需要获取直链链接的文件的fileID
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api("log")
    def log(
        self,
        pageNum: int,
        pageSize: int,
        startTime: int,
        endTime: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """获取直链日志

        Args:
            pageNum (int): 页数
            pageSize (int): 分页大小
            startTime (int): 开始时间
            endTime (int): 结束时间
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api("disable")
    def disable(
        self,
        fileID: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """禁用直链

        Args:
            fileID (int): 禁用直链空间的文件夹的fileID
            skip (bool): 是否跳过响应数据的模式校验
        """



from __future__ import annotations

from typing import Optional

from .utils.api import Auth
from .utils.baseapiclient import BaseApiClient, auto_args_call_api
from .utils.checkdata import DataResponse


class Offline(BaseApiClient):
    def __init__(self, auth: Optional[Auth] = None) -> None:
        super().__init__(filepath="offline", auth=auth)

    @auto_args_call_api()
    def download(
        self,
        url: str,
        fileName: Optional[str] = None,
        dirID: Optional[int] = None,
        callBackUrl: Optional[str] = None,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """
        创建离线下载任务


        Args:
            url (str): 下载链接
            fileName (str, optional): 自定义文件名称 (需携带图片格式,支持格式:png, gif, jpeg, tiff, webp,jpg,tif,svg,bmp)
            dirID (int, optional): 选择下载到指定目录ID.  示例:10023, 注:不支持下载到根目录,默认会下载到名为"来自:离线下载"的目录中
            callBackUrl (str, optional): 回调地址,当文件下载成功或者失败,均会通过回调地址通知.
            skip (bool): 是否跳过响应数据的模式校验
        """

    @auto_args_call_api()
    def process(
        self,
        taskID: int,
        skip: bool = False,
    ) -> DataResponse:  # type: ignore
        """
        获取离线下载进度

        Args:
            taskID (int): 离线下载任务ID
            skip (bool): 是否跳过响应数据的模式校验
        """



import shutil
from pathlib import Path

from cpan123 import Pan123openAPI

pan123 = Pan123openAPI()


#### 上传目录
def test_upload_dir():
    dirname = Path("tdata")
    pan123.upload_dir(dirname)
    print("上传完成")


#### 列出目录下的文件
def test_list_files():
    dirname = Path("/tdata")
    files = pan123.list_files(dirname)
    print(files)


#### 下载目录
def test_download_dir():
    dirname = Path("/tdata")
    pan123.download_dir(dirname, "tdata1")
    print("下载完成")


#### 删除目录
def test_delete_dir():
    dirname = Path("/tdata")
    pan123.delete_dir(dirname)
    print("删除完成")


#### 删除本地的目录
def test_delete_local_dir():
    dirname = Path("tdata1")
    if dirname.exists():
        shutil.rmtree(dirname)
        print("删除本地目录完成")
    else:
        print("目录不存在")


##### 默认找的是目录,返回, id,和 该 id 的详细信息
def test_find():
    fileId, fileId_item = pan123.find_file_id_by_path("/教科版高中物理")
    print("---" * 10)
    print("文件ID:", fileId)
    print("文件ID详细信息:", fileId_item)
    print("---" * 10)


def test_find2():
    # 找文件
    file = "/读秀相关工具/独秀机器人/Robot0309.zip"
    fileId, fileId_item = pan123.find_file_id_by_path(file, is_dir=False)
    print("---" * 10)
    print("文件ID:", fileId)
    print("文件ID详细信息:", fileId_item)
    print("---" * 10)


def test_find3():
    ## 实际是文件,但找的是目录,则会抛出异常
    file = "/读秀相关工具/独秀机器人/Robot0309.zip"
    error = False
    try:
        fileId, fileId_item = pan123.find_file_id_by_path(file, is_dir=True)
        print("---" * 10)
        print("文件ID:", fileId)
        print("文件ID详细信息:", fileId_item)
        print("---" * 10)
    except Exception as e:
        print(e)
        error = True
    assert error, "没有抛出异常"


if __name__ == "__main__":
    pass
    # test_delete_dir()
    test_upload_dir()
    # test_list_files()
    # test_download_dir()
    # test_find()
    # test_find2()
    # test_find3()
